/**
 * Copyright (C) oStorybook Team
 *
 * This program is free software, see <http://www.gnu.org/licenses/>.
 *
 * Parts of this code are from the SHEF project developed and published by Bob Tantlinger.
 */
package shef.actions;

import i18n.I18N;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import javax.swing.Action;
import javax.swing.JEditorPane;
import javax.swing.KeyStroke;
import resources.icons.ICONS;
import shef.editors.wys.WysiwygEditor;

/**
 * @author Bob Tantlinger
 *
 */
public class EditCopyAction extends BasicEditAction {

	private static final String TT = "EditCopyAction";

	public EditCopyAction(WysiwygEditor editor) {
		super(editor, "");
		putValue(Action.NAME, I18N.getMsg("shef.copy"));
		setSmallIcon(ICONS.K.EDIT_COPY);
		setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_DOWN_MASK));
		setMnemonic("copy");
		addShouldBeEnabledDelegate((Action a) -> {
			JEditorPane ed = getCurrentEditor();
			return ed != null && ed.getSelectionStart() != ed.getSelectionEnd();
		});
		setShortDescription(I18N.getMsg("shef.copy_desc"));
	}

	@Override
	protected void doEdit(ActionEvent e, JEditorPane editor) {
		//LOG.trace(TT + ".doEdit(e=" + e.toString() + ", editor)");
		editor.copy();
	}

	@Override
	protected void contextChanged() {
		super.contextChanged();
		this.updateEnabledState();
	}

}
