/**
 * Copyright (C) oStorybook Team
 *
 * This program is free software, see <http://www.gnu.org/licenses/>.
 *
 * Parts of this code are from the SHEF project developed and published by Bob Tantlinger.
 */
package shef.actions;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import javax.swing.JEditorPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import shef.dialogs.TextFinderDialog;
import shef.editors.wys.WysiwygEditor;
import i18n.I18N;

public class FindReplaceAction extends BasicEditAction {

	private boolean isReplaceTab;
	private TextFinderDialog dialog;

	public FindReplaceAction(WysiwygEditor editor, boolean isReplace) {
		super(editor, null);
		if (isReplace) {
			putValue(NAME, I18N.getMsg("shef.replace_"));
			putValue(MNEMONIC_KEY, I18N.getMnem("shef.replace_"));
		} else {
			putValue(NAME, I18N.getMsg("shef.find_"));
			putValue(MNEMONIC_KEY, I18N.getMnem("shef.find_"));
			putValue(ACCELERATOR_KEY,
					KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_DOWN_MASK));
		}

		isReplaceTab = isReplace;
	}

	@Override
	protected void doEdit(ActionEvent e, JEditorPane textComponent) {
		Component c = SwingUtilities.getWindowAncestor(textComponent);
		if (dialog == null) {
			if (c instanceof Frame) {
				if (isReplaceTab) {
					dialog = new TextFinderDialog((Frame) c, textComponent, TextFinderDialog.REPLACE);
				} else {
					dialog = new TextFinderDialog((Frame) c, textComponent, TextFinderDialog.FIND);
				}
			} else if (c instanceof Dialog) {
				if (isReplaceTab) {
					dialog = new TextFinderDialog((Dialog) c, textComponent, TextFinderDialog.REPLACE);
				} else {
					dialog = new TextFinderDialog((Dialog) c, textComponent, TextFinderDialog.FIND);
				}
			} else {
				return;
			}
		}
		if (!dialog.isVisible()) {
			dialog.show((isReplaceTab) ? TextFinderDialog.REPLACE : TextFinderDialog.FIND);
		}
	}

	@Override
	protected void updateContextState(JEditorPane editor) {
		if (dialog != null) {
			dialog.setJTextComponent(editor);
		}
	}

}
