/*
 * Copyright (C) oStorybook Team
 *
 * This program is free software, see <http://www.gnu.org/licenses/>.
 *
 * Parts of this code are from the NovaWorx project. Other parts are
 * from the SHEF project developed and published by Bob Tantlinger.
 */
package shef.editors.wys;

import i18n.I18N;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.undo.UndoManager;
import jortho.SpellChecker;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import shef.ShefEditor;
import shef.actions.ActionDefault;
import shef.actions.CompoundUndoManager;
import shef.actions.EditCopyAction;
import shef.actions.EditPasteAction;
import shef.actions.EditPasteFormattedAction;
import shef.actions.manager.ActionList;
import shef.editors.AbstractPanel;
import static shef.editors.wys.HTMLCadratinAction.*;
import shef.tools.HtmlUtils;
import storybook.App;
import storybook.Const;
import storybook.Const.Spelling;
import storybook.tools.TextUtil;
import storybook.tools.html.CSS;
import storybook.tools.html.Html;
import storybook.tools.net.Net;
import storybook.tools.swing.FontUtil;
import storybook.tools.swing.SwingUtil;
import storybook.tools.swing.js.JSLabel;
import storybook.tools.swing.js.JSMenuScroller;
import storybook.tools.synonyms.Synonyms;
import storybook.tools.synonyms.Word;
import storybook.ui.MIG;

/**
 * initial adaptation by favdb (2020)
 *
 */
public class WysiwygEditor extends AbstractPanel implements HyperlinkListener {

	//private static final String TT = "WysiwygEditor";
	private static final String INVALID_TAGS[] = {"html", "head", "body", "title"};
	private JEditorPane wysEditor;
	private JComboBox cbFontFamily;
	private JComboBox cbParagraph;
	private JPopupMenu wysPopupMenu;
	private FocusListener focusHandler = new FocusHandler();
	public DocumentListener textChanged = new TextChangedHandler();
	private ActionListener fontChanged = new FontChangeHandler();
	private ActionListener cbParagraphHandler = new ParagraphComboHandler();
	private CaretListener caretHandler = new CaretHandler();
	private MouseListener popupHandler = new PopupHandler();
	private boolean isTextChanged = false;
	private int reduced = 2;//0= no toolbar, 1=reduced toolbar, 2=full toolbar
	private int dlgLang = ShefEditor.DLG_EN;//for dialog open/close, 0=all, 1=english only, 2=french only, 3=none
	private ShefEditor editor;
	private JPanel statusbar, shortcut;
	private JSLabel lbStatus;
	private JToolBar tb1;
	private JToolBar tb2;
	private JToolBar linksBar;
	private JButton btMod;
	private int maxLen = 32768;
	private JProgressBar progress;
	private boolean tb2Show = true;
	private JButton btTb2;
	private JToolBar tb11;
	private Dimension tbSize;
	private JMenu synMenu = null, antMenu = null;
	private JMenuItem synLook = null, antLook = null;

	public WysiwygEditor(ShefEditor editor, String param) {
		super(editor);
		this.editor = editor;
		if (!param.isEmpty()) {
			String p[] = param.split(",");
			for (String pp : p) {
				if (pp.startsWith("len ")) {
					this.maxLen = Integer.parseInt(pp.substring(pp.indexOf(" ") + 1));
				} else {
					switch (pp.trim().toLowerCase()) {
						case "reduced":
							reduced = ShefEditor.REDUCED;
							break;
						case "full":
							reduced = ShefEditor.FULL;
							break;
						case "none":
							reduced = ShefEditor.NONE;
							break;
						case "lang_en":
							dlgLang = ShefEditor.DLG_EN;
							break;
						case "lang_fr":
							dlgLang = ShefEditor.DLG_FR;
							break;
						case "lang_all":
							dlgLang = ShefEditor.DLG_ALL;
							break;
						case "lang_none":
							dlgLang = ShefEditor.DLG_NONE;
							break;
						default:
							break;
					}
				}
			}
		}
		initUI();
	}

	@Override
	public void setCaretPosition(int pos) {
		//LOG.trace(TT + ".setCaretPosition(pos=\"" + pos + "\")");
		if (wysEditor.getCaretPosition() != pos) {
			try {
				wysEditor.setCaretPosition(pos);
			} catch (Exception ex) {
				wysEditor.setCaretPosition(wysEditor.getDocument().getLength());
			}
		}
	}

	private void initUI() {
		//LOG.trace(TT + ".initUi()");
		setLayout(new BorderLayout());
		setName(ShefEditor.WYSIWYG);
		JPanel top = new JPanel(new BorderLayout());
		top.add(initToolbar(), BorderLayout.NORTH);
		progress = new JProgressBar(0, 32768);
		progress.setBorder(BorderFactory.createEmptyBorder());
		progress.setMaximumSize(new Dimension(32768, 4));
		if (editor.isStatus()) {
			top.add(progress, BorderLayout.SOUTH);
		}
		add(top, BorderLayout.NORTH);
		wysEditor = createWysEditor();
		add(new JScrollPane(wysEditor), BorderLayout.CENTER);
		add(initStatusbar(), BorderLayout.SOUTH);
		statusbar.setVisible(editor.isStatus());
	}

	@SuppressWarnings("unchecked")
	@Override
	public JToolBar initToolbar() {
		//LOG.trace(TT + ".initToolbar()");
		actionList = new ActionList("editor-actions");
		if (toolbar == null) {
			super.initToolbar();
			toolbar.setFocusable(false);
		} else {
			toolbar.removeAll();
		}
		wysPopupMenu = new JPopupMenu();
		wysPopupMenu.add(CompoundUndoManager.ActionUndo);
		wysPopupMenu.add(CompoundUndoManager.ActionRedo);
		if (App.getInstance().dictaphoneHas()) {
			wysPopupMenu.add((new JSeparator()));
			JMenuItem it = new JMenuItem();
			if (!App.getInstance().dictaphoneIsStarted()) {
				it.setText(I18N.getMsg("dictaphone.start"));
				it.setIcon(IconUtil.getIconSmall(ICONS.K.DICTAPHONE_START));
			} else {
				it.setText(I18N.getMsg("dictaphone.stop"));
				it.setIcon(IconUtil.getIconSmall(ICONS.K.DICTAPHONE_STOP));
			}
			it.addActionListener(e -> App.dictaphoneStart(it));
			wysPopupMenu.add(it);
		}
		wysPopupMenu.add((new JSeparator()));
		wysPopupMenu.add(new HTMLElementPropertiesAction(this));
		initHideButtons();
		if (reduced == ShefEditor.FULL) {
			btTb2 = new JButton("▲");
			btTb2.setToolTipText(I18N.getMsg("shef.2line.hide"));
			btTb2.addActionListener(e -> hideTb2());
			toolbar.add(btTb2);
		}
		initTb1();
		initTb2();
		toolbar.add(tb1);
		toolbar.add(tb11, MIG.get(MIG.RIGHT));
		if (reduced != 1) {
			toolbar.add(tb2, MIG.get(MIG.SPAN, MIG.NEWLINE));
		}
		tbSize = toolbar.getSize();
		if (reduced == 0) {
			hideToolbar(tb1);
			hideToolbar(tb2);
			hideToolbar(toolbar);
		}
		return toolbar;
	}

	/**
	 * init tool bar 1
	 */
	@SuppressWarnings("unchecked")
	private void initTb1() {
		//LOG.trace(TT + ".initTb1()");
		tb1 = new JToolBar();
		tb1.setLayout(new MigLayout(MIG.get(MIG.FLOWX, MIG.INS0, MIG.GAP0)));
		tb1.setFloatable(false);
		PropertyChangeListener propLst = (PropertyChangeEvent evt) -> {
			if (evt.getPropertyName().equals("selected") && evt.getNewValue().equals(Boolean.TRUE)) {
				cbParagraph.removeActionListener(cbParagraphHandler);
				cbParagraph.setSelectedItem(evt.getSource());
				cbParagraph.addActionListener(cbParagraphHandler);
			}
		};
		ActionList blockActs = HTMLBlockAction.createBlockElementActionList(this);
		for (Iterator it = blockActs.iterator(); it.hasNext();) {
			Object o = it.next();
			if (o instanceof ActionDefault) {
				((ActionDefault) o).addPropertyChangeListener(propLst);
			}
		}
		tb1.add(new JLabel(I18N.getColonMsg("shef.style")));
		cbParagraph = new JComboBox(toArray(blockActs));
		cbParagraph.setToolTipText(I18N.getMsg("shef.paragraph"));
		cbParagraph.addActionListener(cbParagraphHandler);
		cbParagraph.setRenderer(new ParagraphComboRenderer());
		tb1.add(cbParagraph, MIG.GROWX);
		tb1.add(initButton(actionList, new HTMLInlineAction(this, HTMLInlineAction.EM)));
		if (reduced != 1) {
			tb1.add(new JLabel(I18N.getColonMsg("shef.font")));
			List<String> fonts = new ArrayList<>();
			fonts.add(I18N.getMsg("shef.default"));
			fonts.add("serif");
			fonts.add("sans-serif");
			fonts.add("monospaced");
			GraphicsEnvironment gEnv = GraphicsEnvironment.getLocalGraphicsEnvironment();
			fonts.addAll(Arrays.asList(gEnv.getAvailableFontFamilyNames()));
			cbFontFamily = new JComboBox(fonts.toArray());
			cbFontFamily.setToolTipText(I18N.getMsg("shef.font_desc"));
			cbFontFamily.addActionListener(fontChanged);
			tb1.add(cbFontFamily, MIG.GROWX);

			JButton btFontsize = new JButton(IconUtil.getIconSmall("fontsize"));
			btFontsize.setName("btFontsize");
			btFontsize.setToolTipText(I18N.getMsg("shef.fontsize_desc"));
			btFontsize.addActionListener(e -> popupFontSize(btFontsize));
			tb1.add(btFontsize);

			Action act = new HTMLFontColorAction(this);
			actionList.add(act);

			JButton btColor = new JButton(act);
			btColor.setText("");
			btColor.setEnabled(true);
			tb1.add(btColor);
		}
		// undo
		Action action = CompoundUndoManager.ActionUndo;
		actionList.add(action);
		JButton jb = new JButton(action);
		jb.setText("");
		tb1.add(jb);
		// redo
		action = CompoundUndoManager.ActionRedo;
		actionList.add(action);
		jb = new JButton(action);
		jb.setText("");
		tb1.add(jb);
		//zoom
		tb1.add(initButton(actionList, new Zoom(this, 1)));
		tb1.add(initButton(actionList, new Zoom(this, 0)));
		// view wysiwyg/html
		tb11 = new JToolBar();
		tb11.setLayout(new MigLayout(MIG.get(MIG.FLOWX, MIG.INS0, MIG.GAP0)));
		tb11.setFloatable(false);
		tb11.add(initButton(ShefEditor.SOURCE, "", "source",
				ShefEditor.SOURCE,
				e -> editor.changeTo(ShefEditor.SOURCE)));
		btMod = initButton(ShefEditor.HTML, "", ShefEditor.HTML,
				ShefEditor.HTML,
				e -> editor.changeTo(ShefEditor.HTML));
		tb11.add(btMod);
		if (App.getInstance().dictaphoneHas()) {
			JButton dict = new JButton();
			if (!App.getInstance().dictaphoneIsStarted()) {
				dict.setIcon(IconUtil.getIconSmall(ICONS.K.DICTAPHONE_START));
				dict.setToolTipText(I18N.getMsg("dictaphone.start"));
			} else {
				dict.setIcon(IconUtil.getIconSmall(ICONS.K.DICTAPHONE_STOP));
				dict.setToolTipText(I18N.getMsg("dictaphone.stop"));
			}
			dict.addActionListener(e -> App.getInstance().dictaphoneStart(dict));
			tb1.add(dict);
		}
	}

	public JToolBar getToolBar1() {
		return tb1;
	}

	/**
	 * init tool bar 2
	 */
	private void initTb2() {
		//LOG.trace(TT + ".initTb2()");
		tb2 = new JToolBar();
		tb2.setLayout(new MigLayout(MIG.get(MIG.FLOWX, MIG.INS0, MIG.GAP0)));
		tb2.setFloatable(false);
		if (reduced == 1) {
			tb2 = tb1;
		}
		//character styles
		tb2.add(initButton(actionList, new HTMLInlineAction(this, HTMLInlineAction.BOLD)));
		tb2.add(initButton(actionList, new HTMLInlineAction(this, HTMLInlineAction.ITALIC)));
		tb2.add(initButton(actionList, new HTMLInlineAction(this, HTMLInlineAction.UNDERLINE)));
		if (reduced != 1) {
			tb2.add(initButton(actionList, new HTMLInlineAction(this, HTMLInlineAction.STRIKE)));
			tb2.add(initButton(actionList, new HTMLInlineAction(this, HTMLInlineAction.SUB)));
			tb2.add(initButton(actionList, new HTMLInlineAction(this, HTMLInlineAction.SUP)));
		}
		tb2.addSeparator();
		//lists ol ul
		tb2.add(initButton(actionList, new HTMLBlockAction(this, HTMLBlockAction.OL)));
		tb2.add(initButton(actionList, new HTMLBlockAction(this, HTMLBlockAction.UL)));
		tb2.addSeparator();
		tb2.add(initButton(actionList, new HTMLAlignAction(this, HTMLAlignAction.LEFT)));
		tb2.add(initButton(actionList, new HTMLAlignAction(this, HTMLAlignAction.CENTER)));
		tb2.add(initButton(actionList, new HTMLAlignAction(this, HTMLAlignAction.RIGHT)));
		tb2.add(initButton(actionList, new HTMLAlignAction(this, HTMLAlignAction.JUSTIFY)));
		tb2.addSeparator();
		//links, image, table
		if (reduced != 1) {
			linksBar = new JToolBar();
			linksBar.setFloatable(false);
			linksBar.add(initButton(actionList, new HTMLLinkAction(this)));
			linksBar.add(initButton(actionList, new HTMLImageAction(this)));
			linksBar.add(initButton(actionList, new HTMLTableAction(this)));
			tb2.add(linksBar);
		}
		//special characters (unicode, linebreak, dialog open/close)
		JButton btChar = this.initButton("btChar", "", "cogs", "char_special");
		btChar.addActionListener(e -> popupChar(btChar));
		tb2.add(btChar);
	}

	/**
	 * initialize phantom buttons
	 */
	private void initHideButtons() {
		//LOG.trace(TT + ".initHideButton()");
		shortcut = new JPanel();
		SwingUtil.setForcedSize(shortcut, new Dimension(1, 1));
		shortcut.add(initButton(actionList, new HelpAction(this)));
		shortcut.add(initButton(actionList, new HTMLLineBreakAction(this)));
		shortcut.add(initButton(actionList, new HTMLCadratinAction(this, UNBREAK)));
		shortcut.add(initButton(actionList, new EditCopyAction(this)));
		shortcut.add(initButton(actionList, new EditPasteAction(this)));
		shortcut.add(initButton(actionList, new EditPasteFormattedAction(this)));
		toolbar.add(shortcut);
	}

	public void setHide(boolean hide) {
		//LOG.trace(TT + ".setHide()");
		tb1.setVisible(hide);
		tb2.setVisible(hide);
	}

	public JToolBar getLinksBar() {
		return linksBar;
	}

	public JToolBar getToolBar2() {
		return tb2;
	}

	private boolean tbShow = true;

	public void hideToolbar() {
		tbShow = !tbShow;
		if (tbShow) {
			toolbar.setPreferredSize(tbSize);
		} else {
			hideToolbar(toolbar);
		}
	}

	public void hideToolbar(JToolBar tb) {
		//LOG.trace(TT + ".hideToolbar(tb)");
		Dimension d = new Dimension(0, 0);
		tb.setMaximumSize(d);
		tb.setPreferredSize(d);
	}

	private void hideTb2() {
		tb2Show = !tb2Show;
		if (tb2Show) {
			tb2.setPreferredSize(tb1.getSize());
			tb2.setMaximumSize(tb1.getSize());
			btTb2.setToolTipText(I18N.getMsg("shef.2line.hide"));
		} else {
			tb2.setPreferredSize(new Dimension(0, 0));
			tb2.setMaximumSize(new Dimension(0, 0));
			btTb2.setToolTipText(I18N.getMsg("shef.2line.show"));
		}
		btTb2.setText((tb2Show ? "▲" : "▼"));
	}

	/**
	 * init status bar
	 *
	 * @return
	 */
	private JPanel initStatusbar() {
		//LOG.trace(TT + ".initStatusbar()");
		statusbar = new JPanel(new BorderLayout());
		lbStatus = new JSLabel();
		lbStatus.setFont(FontUtil.getSmallFont());
		statusbar.add(lbStatus, BorderLayout.LINE_END);
		return statusbar;
	}

	/**
	 * refresh status info
	 */
	public void refreshStatus() {
		//LOG.trace(TT + ".refreshStatus()");
		String text = getText();
		Integer txtLen = text.length();
		if (maxLen > 0) {
			Integer pourcent = (int) (maxLen * 0.1);
			int len = maxLen - txtLen;
			if (len < 0) {
				lbStatus.setForeground(Color.red);
				progress.setForeground(Color.red);
			} else if (len < pourcent) {
				lbStatus.setForeground(Color.orange);
				progress.setForeground(Color.orange);
			} else {
				lbStatus.setForeground(Color.black);
				progress.setForeground(Color.green);
			}
			progress.setValue(maxLen - len);
		}
		lbStatus.setText(I18N.getColonMsg("editor.words") + TextUtil.countWords(text) + " (" + txtLen + ")");
	}

	/**
	 * Converts an action list to an array. Any of the null "separators" or sub ActionLists are
	 * ommited from the array.
	 *
	 * @param lst
	 * @return
	 */
	@SuppressWarnings("unchecked")
	private Action[] toArray(ActionList lst) {
		List acts = new ArrayList();
		for (Iterator it = lst.iterator(); it.hasNext();) {
			Object v = it.next();
			if (v != null && v instanceof Action) {
				acts.add(v);
			}
		}
		return (Action[]) acts.toArray(new Action[acts.size()]);
	}

	/**
	 * create de Wysiwyg editor pane
	 *
	 * @return
	 */
	private JEditorPane createWysEditor() {
		//LOG.trace(TT + ".createWysEditor()");
		Font font = App.getInstance().fontGetEditor();
		JEditorPane ed = new JEditorPane();
		ed.setName("wysEditor");
		ed.setFont(font);
		WysiwygEditorKit kit = new WysiwygEditorKit(this);
		ed.setEditorKitForContentType("text/html", kit);
		ed.setContentType("text/html");
		insertHTML(ed, "<p></p>", 0);
		ed.addCaretListener(caretHandler);
		ed.addFocusListener(focusHandler);
		// spell checker, must be before the popup handler
		String spelling = ShefEditor.getSpelling();
		if (!Spelling.none.name().equals(spelling)) {
			SpellChecker.register(ed);
		}
		ed.addMouseListener(popupHandler);
		HTMLDocument document = (HTMLDocument) ed.getDocument();
		CompoundUndoManager cuh = new CompoundUndoManager(document, new UndoManager());
		document.addUndoableEditListener(cuh);
		document.getStyleSheet().addRule(CSS.forEditor());
		ed.addHyperlinkListener(this);
		spellingInit(ed);
		document.addDocumentListener(textChanged);
		return ed;
	}

	/**
	 * inserts html into the wysiwyg editor
	 *
	 * @param editor
	 * @param html
	 * @param location
	 */
	private void insertHTML(JEditorPane editor, String html, int location) {
		try {
			HTMLEditorKit kit = (HTMLEditorKit) editor.getEditorKit();
			Document doc = editor.getDocument();
			StringReader reader = new StringReader(HtmlUtils.jEditorPaneizeHTML(html));
			kit.read(reader, doc, location);
		} catch (IOException | BadLocationException ex) {
			ex.printStackTrace(System.err);
		}
	}

	/**
	 * set text to the Wysiwyg pane
	 *
	 * @param text
	 */
	@Override
	public void setText(String text) {
		//LOG.trace(TT + ".setText(text=\"" + text + "\")");
		String rtext = removeInvalidTags(text);
		wysEditor.setText("");
		insertHTML(wysEditor, rtext, 0);
		refreshStatus();
		CompoundUndoManager.discardAllEdits(wysEditor.getDocument());
	}

	public void changeWord(String word) {
		//LOG.trace(TT + ".setSynonym(word=" + word + ")");
		wysEditor.replaceSelection(word);
		setChange();
	}

	public void lookForSynonyms(MouseEvent evt, Word word) {
		//LOG.trace(TT + ".synonymsLook()");
		SwingUtil.setWaitingCursor(this);
		String target = wysEditor.getSelectedText();
		if (word != null && !word.getMot().isEmpty() && !word.getLemme().isEmpty()) {
			target = word.getLemme();
		}
		List<String> words = Synonyms.lookForSynonyms(wysEditor, target);
		SwingUtil.setDefaultCursor(this);
		if (!words.isEmpty()) {
			JPopupMenu pop = new JPopupMenu();
			JSMenuScroller.setScrollerFor(pop, 15, 200, 0, 0);
			for (String s : words) {
				JMenuItem it = new JMenuItem(s);
				it.addActionListener(e -> changeWord(s));
				pop.add(it);
			}
			JMenuItem it = new JMenuItem(I18N.getMsg("word.synonyms.url_show") + " >");
			it.setToolTipText(Synonyms.SYNONYMS_URL_PROMPT);
			final String sword = target;
			it.addActionListener(e -> Synonyms.showForSynonyms(wysEditor, sword));
			pop.add(it);
			pop.show(evt.getComponent(), evt.getX(), evt.getY());
		}
	}

	public void lookForAntonyms(MouseEvent evt, Word word) {
		//LOG.trace(TT + ".synonymsLook()");
		SwingUtil.setWaitingCursor(this);
		String target = wysEditor.getSelectedText();
		if (word != null && !word.getMot().isEmpty() && !word.getLemme().isEmpty()) {
			target = word.getLemme();
		}
		List<String> words = Synonyms.lookForAntonyms(wysEditor, target);
		SwingUtil.setDefaultCursor(this);
		if (!words.isEmpty()) {
			JPopupMenu pop = new JPopupMenu();
			JSMenuScroller.setScrollerFor(pop, 15, 200, 0, 0);
			for (String s : words) {
				JMenuItem it = new JMenuItem(s);
				it.addActionListener(e -> changeWord(s));
				pop.add(it);
			}
			JMenuItem it = new JMenuItem(I18N.getMsg("word.antonyms.url_show") + " >");
			it.setToolTipText(Synonyms.ANTONYMS_URL_PROMPT);
			final String sword = target;
			it.addActionListener(e -> Synonyms.showForAntonyms(wysEditor, sword));
			pop.add(it);
			pop.show(evt.getComponent(), evt.getX(), evt.getY());
		}
	}

	/**
	 * get the text content of the Wyiwyg pane
	 *
	 * @return
	 */
	public String getText() {
		return Html.removeTag(removeInvalidTags(wysEditor.getText()), "style");
	}

	/**
	 * check if content has changed
	 *
	 * @return : true if content has changed
	 */
	public boolean isTextChanged() {
		return this.isTextChanged;
	}

	/**
	 * removing invalid tags
	 *
	 * @param html
	 * @return
	 */
	private String removeInvalidTags(String html) {
		for (String tag : INVALID_TAGS) {
			html = deleteOccurance(html, "<" + tag + ">");
			html = deleteOccurance(html, "</" + tag + ">");
		}
		return html.trim();
	}

	/**
	 * delete a word from a String
	 *
	 * @param text
	 * @param word
	 * @return
	 */
	private String deleteOccurance(String text, String word) {
		StringBuilder sb = new StringBuilder(text);
		int p;
		while ((p = sb.toString().toLowerCase().indexOf(word.toLowerCase())) != -1) {
			sb.delete(p, p + word.length());
		}
		return sb.toString();
	}

	/**
	 * update the state
	 */
	private void updateState() {
		//LOG.trace(TT + ".updateState()");
		if (reduced != 1) {
			cbFontFamily.removeActionListener(fontChanged);
			String fontName = HtmlUtils.getFontFamily(wysEditor);
			if (fontName == null) {
				cbFontFamily.setSelectedIndex(0);
			} else {
				cbFontFamily.setSelectedItem(fontName);
			}
			cbFontFamily.addActionListener(fontChanged);
		}
		actionList.putContextValueForAll(HTMLTextEditAction.EDITOR, wysEditor);
		actionList.updateEnabledForAll();
	}

	/**
	 * get the Wysiwyg editor pane
	 *
	 * @return
	 */
	public JEditorPane getEditor() {
		return this.wysEditor;
	}

	public void setEditable(boolean b) {
		wysEditor.setEditable(b);
		btMod.setIcon(IconUtil.getIconSmall(b ? "html" : "pencil"));
		btMod.setToolTipText(I18N.getMsg("shef." + (b ? "html" : "html.normal")));
	}

	@Override
	public void hyperlinkUpdate(HyperlinkEvent evt) {
		if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED
				&& !evt.getDescription().startsWith("#")) {
			Net.openUrl(evt);
		}
	}

	public ActionList getActionList() {
		return actionList;
	}

	public void spellingInit(JEditorPane ed) {
		// spell checker
		String spelling = ShefEditor.getSpelling();
		if (!Const.Spelling.none.name().equals(spelling)) {
			wysPopupMenu.addSeparator();
			wysPopupMenu.add(SpellChecker.createCheckerMenu());
			wysPopupMenu.add(SpellChecker.createLanguagesMenu());
		}
	}

	public void setMaxLen(int maxLen) {
		//LOG.trace(TT + ".setMaxLen(maxLen=\"" + maxLen + "\")");
		this.maxLen = maxLen;
		refreshStatus();
	}

	public void setChange() {
		this.isTextChanged = true;
	}

	/**
	 * remove specific tag String from the HTML code
	 *
	 * @param tag
	 */
	public void removeTag(String tag) {
		//LOG.trace(TT + ".removeTag(tag=\"" + tag + "\")");
		String html = wysEditor.getText();
		if (html.contains(tag)) {
			html = html.replace(tag, "");
			wysEditor.setText(html);
			setChange();
		}
	}

	private void addSynonyms(MouseEvent evt, Word word) {
		if (synMenu != null) {
			wysPopupMenu.remove(synMenu);
		}
		synMenu = null;
		if (word != null) {
			List<String> synonyms = word.getSynonyms();
			if (!synonyms.isEmpty()) {
				synMenu = new JMenu(I18N.getMsg("word.synonyms"));
				if (!word.getLemme().isEmpty()) {
					JLabel lb = new JLabel("= " + word.getLemme());
					lb.setIcon(IconUtil.getIconSmall(ICONS.K.AR_RIGHT));
					lb.setFont(FontUtil.getBoldFont());
					lb.setForeground(Color.blue);
					synMenu.add(lb);
				}
				int n = 10;
				for (String s : synonyms) {
					JMenuItem it = new JMenuItem(s);
					it.addActionListener(e -> changeWord(s));
					synMenu.add(it);
					if (n-- < 0) {
						break;
					}
				}
				wysPopupMenu.add(synMenu);
			}
		}
		addSynonymsLook(evt, word);
	}

	private void addSynonymsLook(MouseEvent evt, Word word) {
		if (synLook != null) {
			wysPopupMenu.remove(synLook);
		}
		synLook = null;
		if (!App.preferences.getWorkOffline()) {
			String url = Synonyms.SYNONYMS_URL;
			if (!url.isEmpty() && !url.startsWith("!")) {
				synLook = new JMenuItem(I18N.getMsg("word.synonyms.look"));
				synLook.setToolTipText(Synonyms.SYNONYMS_URL_PROMPT);
				synLook.addActionListener(e -> lookForSynonyms(evt, word));
				if (synMenu != null) {
					synMenu.add(synLook);
				} else {
					wysPopupMenu.add(synLook);
				}
			}
		}
	}

	private void addAntonyms(MouseEvent evt, Word word) {
		if (antMenu != null) {
			wysPopupMenu.remove(antMenu);
		}
		antMenu = null;
		if (word != null) {
			List<String> words = word.getAntonyms();
			if (!words.isEmpty()) {
				antMenu = new JMenu(I18N.getMsg("word.antonyms"));
				if (!word.getLemme().isEmpty()) {
					JLabel lb = new JLabel("# " + word.getLemme());
					lb.setIcon(IconUtil.getIconSmall(ICONS.K.AR_RIGHT));
					lb.setFont(FontUtil.getBoldFont());
					lb.setForeground(Color.blue);
					antMenu.add(lb);
				}
				int n = 10;
				for (String s : words) {
					JMenuItem it = new JMenuItem(s);
					it.addActionListener(e -> changeWord(s));
					antMenu.add(it);
					if (n-- < 0) {
						break;
					}
				}
				wysPopupMenu.add(antMenu);
			} else {
				antMenu = null;
			}
		}
		addAntonymsLook(evt, word);
	}

	private void addAntonymsLook(MouseEvent evt, Word word) {
		if (antLook != null) {
			wysPopupMenu.remove(antLook);
		}
		antLook = null;
		if (!App.preferences.getWorkOffline()) {
			String url = Synonyms.ANTONYMS_URL;
			if (!url.isEmpty() && !url.startsWith("!")) {
				antLook = new JMenuItem(I18N.getMsg("word.antonyms.look"));
				antLook.setToolTipText(Synonyms.ANTONYMS_URL_PROMPT);
				antLook.addActionListener(e -> lookForAntonyms(evt, word));
				if (antMenu != null) {
					antMenu.add(antLook);
				} else {
					wysPopupMenu.add(antLook);

				}
			}
		}
	}

	private void popupChar(JButton bt) {
		JPopupMenu pop = new JPopupMenu();
		pop.add(initMenuItem(new HTMLLineBreakAction(this)));
		pop.add(initMenuItem(new HTMLCadratinAction(this, UNBREAK)));
		pop.add(initMenuItem(new HTMLUnicodeAction(this)));
		pop.add(initMenuItem(new HTMLCadratinAction(this, CADRATIN)));
		if (dlgLang == ShefEditor.DLG_ALL || dlgLang == ShefEditor.DLG_EN) {
			pop.add(initMenuItem(new HTMLCadratinAction(this, DLG_OPEN_EN)));
			pop.add(initMenuItem(new HTMLCadratinAction(this, DLG_CLOSE_EN)));
		}
		if (dlgLang == ShefEditor.DLG_ALL || dlgLang == ShefEditor.DLG_FR) {
			pop.add(initMenuItem(new HTMLCadratinAction(this, DLG_OPEN_FR)));
			pop.add(initMenuItem(new HTMLCadratinAction(this, DLG_CLOSE_FR)));
		}
		pop.show(bt, bt.getSize().height, 0);
	}

	private void popupFontSize(JButton bt) {
		//LOG.trace(TT + ".popupFontSize(bt=" + bt.getName() + ")");
		JPopupMenu pop = new JPopupMenu();
		for (int i = 0; i < HTMLFontSizeAction.FONT_SIZES.length; i++) {
			pop.add(initMenuItem(new HTMLFontSizeAction(this, i)));
		}
		pop.show(bt, bt.getSize().height, 0);
	}

	/**
	 * handle listener for the caret
	 */
	private class CaretHandler implements CaretListener {

		@Override
		public void caretUpdate(CaretEvent e) {
			//LOG.trace(TT + "::" + "CaretHandler" + ".caretUpdate(e=\"" + e.toString() + "\")");
			updateState();
		}
	}

	private class PopupHandler extends MouseAdapter {

		@Override
		public void mousePressed(MouseEvent e) {
			checkForPopupTrigger(e);
		}

		@Override
		public void mouseReleased(MouseEvent e) {
			checkForPopupTrigger(e);
		}

		private void checkForPopupTrigger(MouseEvent evt) {
			if (evt.isPopupTrigger()) {
				if (evt.getSource() == wysEditor) {
					String selectedText = wysEditor.getSelectedText();
					if (selectedText != null && !selectedText.isEmpty()) {
						//LOG.trace("selectedText=\"" + selectedText + "\"");
						Word word = Synonyms.findWord(selectedText);
						if (word != null && !word.getMot().equals(selectedText)) {
							word.setLemme(word.getMot());
							word.setMot(selectedText);
						}
						addSynonyms(evt, word);
						addAntonyms(evt, word);
					}
				} else {
					return;
				}
				wysPopupMenu.show(evt.getComponent(), evt.getX(), evt.getY());
			}
		}
	}

	private class FocusHandler implements FocusListener {

		@Override
		public void focusGained(FocusEvent e) {
			//LOG.trace(TT + "FocusHandler(e=" + e.toString() + ")");
			if (e.getComponent() instanceof JEditorPane) {
				JEditorPane ed = (JEditorPane) e.getComponent();
				CompoundUndoManager.updateUndo(ed.getDocument());
				updateState();
			}
		}

		@Override
		public void focusLost(FocusEvent e) {
			if (e.getComponent() instanceof JEditorPane) {
			}
		}
	}

	private class TextChangedHandler implements DocumentListener {

		@Override
		public void insertUpdate(DocumentEvent e) {
			textChanged();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			textChanged();
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			textChanged();
		}

		private void textChanged() {
			//LOG.trace(TT + "::TextChangedHandler.textChanged()");
			isTextChanged = true;
			refreshStatus();
		}
	}

	private class ParagraphComboHandler implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == cbParagraph) {
				Action a = (Action) (cbParagraph.getSelectedItem());
				a.actionPerformed(e);
			}
		}
	}

	private class ParagraphComboRenderer extends DefaultListCellRenderer {

		@Override
		public Component getListCellRendererComponent(JList list, Object value, int index,
				boolean isSelected, boolean cellHasFocus) {
			if (value instanceof Action) {
				value = ((Action) value).getValue(Action.NAME);
			}
			return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		}
	}

	private class FontChangeHandler implements ActionListener {

		@Override
		public void actionPerformed(ActionEvent e) {
			if (e.getSource() == cbFontFamily) {
				HTMLDocument document = (HTMLDocument) wysEditor.getDocument();
				CompoundUndoManager.beginCompoundEdit(document);
				if (cbFontFamily.getSelectedIndex() != 0) {
					HtmlUtils.setFontFamily(wysEditor,
							cbFontFamily.getSelectedItem().toString());
				} else {
					HtmlUtils.setFontFamily(wysEditor, null);
				}
				CompoundUndoManager.endCompoundEdit(document);
			}
		}

		public void itemStateChanged(ItemEvent e) {
			// empty
		}
	}
}
