/*
 Storybook: Open Source software for novelists and authors.
 Copyright (C) 2008 - 2012 Martin Mustun

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.action;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JToolBar;
import storybook.App;
import storybook.Pref.PrefFile;
import storybook.ctrl.ActKey;
import storybook.model.H2File;
import storybook.model.book.Book;
import storybook.tools.LOG;
import storybook.tools.file.EnvUtil;
import storybook.ui.MainFrame;
import storybook.ui.MainMenu;

/**
 * @author martin
 *
 */
public class SbActionManager implements PropertyChangeListener {

	private ActionHandler actionHandler;
	private ActionManager actionManager;
	private final MainFrame mainFrame;
	private MainMenu mainMenu;

	public SbActionManager(MainFrame mainFrame) {
		this.mainFrame = mainFrame;
	}

	public void init() {
		//App.trace("SbActionManager.init()");
		initActions();
		initUiFactory();
		if (mainFrame.isBlank()) {
			mainMenu.setMenuForBlank();
		}
	}

	private void initActions() {
		//App.trace("SbActionManager.initActions()");
		actionManager = new ActionManager();
		ActionManager.setInstance(actionManager);
		registerActions();
	}

	private void registerActions() {
		//App.trace("SbActionManager.registerActions()");
		actionHandler = new ActionHandler(mainFrame);
		//actionManager.registerCallback("save-command", actionHandler, "handleSave");
	}

	private void initUiFactory() {
		//App.trace("SbActionManager.initUiFactory()");
		mainMenu = new MainMenu(mainFrame);
		JMenuBar menubar = mainMenu.menuBar;
		if (menubar != null) {
			reloadRecentMenu();
			//reloadPartMenu();
			reloadWindowMenu();
			mainFrame.setJMenuBar(menubar);
		} else {
			LOG.err("*** General error : unable to load main menu");
		}
		JToolBar toolBar = mainMenu.toolBar;
		if (toolBar != null) {
			toolBar.setName("MainToolbar");
			mainFrame.setMainToolBar(toolBar);
		}
		mainFrame.invalidate();
		mainFrame.validate();
		mainFrame.pack();
		mainFrame.repaint();
	}

	public void reloadWindowMenu() {
		//App.trace("SbActionManager.reloadWindowMenu(" + menubar.getName() + ")");
		JMenu miLoad = mainMenu.windowLoadLayout;
		miLoad.removeAll();
		File dir = new File(EnvUtil.getPrefDir().getAbsolutePath());
		File[] files = dir.listFiles();
		if (files == null || files.length < 1) {
			LOG.log("** Pref dir is empty");
			return;
		}
		for (File file : files) {
			if (file.isFile()) {
				String name = file.getName();
				if (name.endsWith(".layout")) {
					String str = name.substring(0, name.lastIndexOf(".layout"));
					if (str.equals("_internal_last_used_layout_")
							|| str.equals("LastUsedLayout")) {
						continue;
					}
					LoadDockingLayoutAction act = new LoadDockingLayoutAction(mainFrame, str);
					JMenuItem item = new JMenuItem(act);
					item.setFont(App.getInstance().fontGetDefault());
					miLoad.add(item);
				}
			}
		}
	}

	public void reloadRecentMenu() {
		//App.trace("SbActionManager.reloadRecentMenu(" + menubar.getName() + ")");
		JMenu miRecent = mainMenu.getMenuFileOpenRecent();
		miRecent.removeAll();
		App.preferences.recentFilesLoad();
		for (PrefFile p : App.preferences.recentFiles) {
			File f = new File(p.file);
			if (!f.exists()) {
				continue;
			}
			OpenFileAction act = new OpenFileAction(p.title, new H2File(p.file));
			JMenuItem item = new JMenuItem(act);
			//item.setFont(App.getInstance().fontGetDefault());
			miRecent.add(item);
		}
		miRecent.addSeparator();
		JMenuItem item = new JMenuItem(new ClearRecentFilesAction(actionHandler));
		item.setFont(App.getInstance().fontGetDefault());
		miRecent.add(item);
	}

	public void reloadMenuToolbar() {
		if (mainMenu != null) {
			mainMenu.reloadToolbar();
		}
	}

	public ActionHandler getActionController() {
		//App.trace("SbActionManager.getActionController()");
		return actionHandler;
	}

	public ActionManager getActionManager() {
		//App.trace("SbActionManager.getActionManager()");
		return actionManager;
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		//App.trace("SbActionManager.propertyChange(" + evt.getPropertyName() + "::" + evt.getNewValue() + ")");
		ActKey act = new ActKey(evt);
		if (!Book.TYPE.PART.compare(act.type)) {
			return;
		}
		reloadMenuToolbar();
	}

	public ActionHandler getActionHandler() {
		//App.trace("SbActionManager.getActionHandler()");
		return actionHandler;
	}

	public MainMenu getMainMenu() {
		return (mainMenu);
	}

	public void enablePaste() {
		mainMenu.enablePaste();
	}

}
