/*
 * Copyright (C) 2019 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.edit;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import javax.swing.JTextField;
import storybook.Const;
import static storybook.ui.Ui.initIntegerField;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Gender;
import storybook.tools.StringUtil;
import storybook.ui.dialog.chooser.ImageChooserPanel;

/**
 *
 * @author favdb
 */
public class EditGender extends AbstractEditor {

	private JTextField tfChild;
	private JTextField tfAdo;
	private JTextField tfAdult;
	private JTextField tfRetire;
	private ImageChooserPanel tfIcon;

	public EditGender(Editor m, AbstractEntity e) {
		super(m, e, "110");
		initAll();
	}

	@Override
	public void initUpper() {
		Gender gender = (Gender) entity;
		tfChild = initIntegerField(panel, "gender.childhood", 5, gender.getChildhood(), !MANDATORY, !INFO);
		tfAdo = initIntegerField(panel, "gender.adolescence", 5, gender.getAdolescence(), !MANDATORY, !INFO);
		tfAdult = initIntegerField(panel, "gender.adulthood", 5, gender.getAdulthood(), !MANDATORY, !INFO);
		tfRetire = initIntegerField(panel, "gender.retirement", 5, gender.getRetirement(), !MANDATORY, !INFO);
		tfIcon = initIconChooser(panel, "icon.file", gender.getIcone(), !MANDATORY);
	}

	@Override
	public boolean verifier() {
		resetError();
		int child = 0, ado = 0, adult = 0, retire = 0;
		if (!StringUtil.isNumeric(tfChild.getText())) {
			errorMsg(tfChild, Const.ERROR_NOTNUMERIC);
		} else {
			child = Integer.parseInt(tfChild.getText());
		}
		if (!StringUtil.isNumeric(tfAdo.getText())) {
			errorMsg(tfAdo, Const.ERROR_NOTNUMERIC);
		} else {
			ado = Integer.parseInt(tfAdo.getText());
		}
		if (!StringUtil.isNumeric(tfAdult.getText())) {
			errorMsg(tfAdult, Const.ERROR_NOTNUMERIC);
		} else {
			adult = Integer.parseInt(tfAdult.getText());
		}
		if (!StringUtil.isNumeric(tfRetire.getText())) {
			errorMsg(tfRetire, Const.ERROR_NOTNUMERIC);
		} else {
			retire = Integer.parseInt(tfRetire.getText());
		}
		if (child >= ado) {
			errorMsg(tfAdo, Const.ERROR_WRONG);
		}
		if (ado >= adult) {
			errorMsg(tfAdult, Const.ERROR_WRONG);
		}
		if (adult >= retire) {
			errorMsg(tfRetire, Const.ERROR_WRONG);
		}
		return (msgError.isEmpty());
	}

	@Override
	public void apply() {
		Gender gender = (Gender) entity;
		gender.setName(tfName.getText());
		gender.setChildhood(Integer.parseInt(tfChild.getText()));
		gender.setAdolescence(Integer.parseInt(tfAdo.getText()));
		gender.setAdulthood(Integer.parseInt(tfAdult.getText()));
		gender.setRetirement(Integer.parseInt(tfRetire.getText()));
		gender.setIcone(tfIcon.getIconFile());
		super.apply();
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

}
