package storybook.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import org.hibernate.Session;
import storybook.ctrl.ActKey;
import storybook.ctrl.Ctrl;
import static storybook.ctrl.Ctrl.PROPS.*;
import storybook.model.book.Book;
import storybook.model.hbn.SbSessionFactory;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.tools.LOG;
import storybook.ui.MainFrame;

public abstract class AbstractModel {

	protected PropertyChangeSupport propertyChangeSupport;
	protected SbSessionFactory sessionFactory;
	MainFrame mainFrame;
	String name;

	public AbstractModel() {
		sessionFactory = new SbSessionFactory();
	}

	public AbstractModel(MainFrame m) {
		mainFrame = m;
		propertyChangeSupport = new PropertyChangeSupport(this);
		sessionFactory = new SbSessionFactory();
	}

	public String getName() {
		return (name);
	}

	public abstract void fireAgain();

	public void initSession(String dbName) {
		//App.trace("AbstractModel.initSession("+dbName+")");
		sessionFactory.init(dbName);
	}

	public void initDefault() {
		fireAgain();
	}

	public Session beginTransaction() {
		//App.trace("AbstractModel.beginTransaction()");
		Session session = sessionFactory.getSession();
		session.beginTransaction();
		return session;
	}

	public Session getSession() {
		return sessionFactory.getSession();
	}

	public void commit() {
		Session session = sessionFactory.getSession();
		session.getTransaction().commit();
	}

	public void addPropertyChangeListener(PropertyChangeListener l) {
		//App.trace("AbstractModel.addPropertyChangeListener("+l.toString()+")");
		propertyChangeSupport.addPropertyChangeListener(l);
	}

	public void removePropertyChangeListener(PropertyChangeListener l) {
		propertyChangeSupport.removePropertyChangeListener(l);
	}

	protected void firePropertyChange(Book.TYPE type) {
		String prop = new ActKey(type, INIT).toString();
		firePropertyChange(prop, null, EntityUtil.findEntities(mainFrame, type));
	}

	protected void firePropertyChange(Book.TYPE type, Ctrl.PROPS cmd, AbstractEntity old, AbstractEntity value) {
		/*App.trace("AbstractModel.firePropertyChange("
				+ "type="		+ type.name()
				+ ", cmd="		+ cmd.name()
				+ ", old="		+ (old != null ? old.toString() : "null")
				+ ", value="	+ (value != null ? value.toString() : "null") + ")");*/
		String prop = new ActKey(type, cmd).toString();
		try {
			propertyChangeSupport.firePropertyChange(prop, old, value);
		} catch (Exception ex) {
			LOG.err(prop, ex);
		}
	}

	protected void firePropertyChange(String prop, Object oldValue, Object newValue) {
		/*App.trace("AbstractModel.firePropertyChange(" + "prop=" + prop
				+ ", oldValue=" + (oldValue != null ? oldValue.toString() : "null")
				+ ", newValue=" + (newValue != null ? newValue.toString() : "null") + ")");*/
		try {
			propertyChangeSupport.firePropertyChange(prop, oldValue, newValue);
		} catch (Exception ex) {
			LOG.err(prop, ex);
		}
	}

	public SbSessionFactory getSessionFactory() {
		return sessionFactory;
	}

	public void editEntity(AbstractEntity entity) {
		//App.trace("AbstractModel.editEntity("+entity.toString()+")");
		mainFrame.showEditorAsDialog(entity);
	}

	public void closeSession() {
		sessionFactory.closeSession();
	}
}
