/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun, 2015 FaVdB

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model;

import i18n.I18N;
import java.awt.Component;
import java.awt.Dimension;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import org.hibernate.Session;
import org.hibernate.exception.ConstraintViolationException;
import org.miginfocom.swing.MigLayout;
import storybook.ctrl.ActKey;
import storybook.ctrl.Ctrl;
import static storybook.ctrl.Ctrl.PROPS.*;
import storybook.exim.EXIM;
import storybook.model.book.Book;
import static storybook.model.book.Book.TYPE.*;
import storybook.model.hbn.dao.AttributeDAO;
import storybook.model.hbn.dao.CategoryDAO;
import storybook.model.hbn.dao.ChapterDAO;
import storybook.model.hbn.dao.EndnoteDAO;
import storybook.model.hbn.dao.EpisodeDAO;
import storybook.model.hbn.dao.EventDAO;
import storybook.model.hbn.dao.GenderDAO;
import storybook.model.hbn.dao.IdeaDAO;
import storybook.model.hbn.dao.InternalDAO;
import storybook.model.hbn.dao.ItemDAO;
import storybook.model.hbn.dao.ItemlinkDAO;
import storybook.model.hbn.dao.LocationDAO;
import storybook.model.hbn.dao.MemoDAO;
import storybook.model.hbn.dao.PartDAO;
import storybook.model.hbn.dao.PersonDAO;
import storybook.model.hbn.dao.PlotDAO;
import storybook.model.hbn.dao.RelationDAO;
import storybook.model.hbn.dao.SceneDAO;
import storybook.model.hbn.dao.StrandDAO;
import storybook.model.hbn.dao.TagDAO;
import storybook.model.hbn.dao.TaglinkDAO;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Attribute;
import storybook.model.hbn.entity.Category;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Episode;
import storybook.model.hbn.entity.Event;
import storybook.model.hbn.entity.Gender;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Internal;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Itemlink;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Memo;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Plot;
import storybook.model.hbn.entity.Relationship;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Strand;
import storybook.model.hbn.entity.Tag;
import storybook.model.hbn.entity.Taglink;
import storybook.model.state.SceneStatus;
import storybook.tools.DateUtil;
import storybook.tools.LOG;
import storybook.tools.StringUtil;
import storybook.tools.swing.ColorUtil;
import storybook.tools.swing.FontUtil;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.SbView;
import storybook.ui.SbView.VIEWNAME;
import storybook.ui.panel.book.BookPanel;
import storybook.ui.panel.chrono.ChronoPanel;
import storybook.ui.panel.manage.Manage;
import storybook.ui.panel.reading.ReadingPanel;

/**
 * @author martin
 *
 */
public class Model extends AbstractModel {

	private final static String TT = "Model";

	private JFrame progressFrame;
	private JProgressBar progressBar;

	public Model(MainFrame m) {
		super(m);
		name = "book";
	}

	public Model() {
		super();
		name = "book";
	}

	public synchronized void initBaseEntities(String title) {
		//App.trace("Model.initBaseEntities()");
		Session session = beginTransaction();

		// default strand
		Strand strand = new Strand(1,
				I18N.getMsg("strand.name.init_value"),
				I18N.getMsg("strand.abbr.init_value"),
				ColorUtil.PALETTE.LIGHT_BLUE.getRGB(), "");
		session.save(strand);

		// default genders
		Gender male = new Gender(I18N.getMsg("person.gender.male"), 6, 12, 18, 65);
		session.save(male);
		Gender female = new Gender(I18N.getMsg("person.gender.female"), 6, 12, 18, 65);
		session.save(female);

		// default categories
		Category major = new Category(1, I18N.getMsg("category.central_character"), null);
		session.save(major);
		Category minor = new Category(2, I18N.getMsg("category.minor_character"), null);
		session.save(minor);

		session.getTransaction().commit();
	}

	public static int computeObjectiveChars(int nature, int nb) {
		if (nature == 0 || nb == 0) {
			return 0;
		}
		String s = I18N.getMsg("book.nature." + nature);
		if (s.startsWith("!")) {
			return 0;
		}
		if (!s.contains(":")) {
			return 0;
		}
		s = s.substring(s.indexOf(":") + 1);
		if (!StringUtil.isNumeric(s)) {
			return 0;
		}
		return Integer.parseInt(s) / nb;
	}

	public synchronized void initEntities(int nature, int nbParts, int nbChapters, Date objective) {
		//App.trace("Model.initEntities()");
		Session session = beginTransaction();
		// default strand
		Strand strand = new Strand(1,
				I18N.getMsg("strand.name.init_value"),
				I18N.getMsg("strand.abbr.init_value"),
				ColorUtil.PALETTE.LIGHT_BLUE.getRGB(), "");
		session.save(strand);
		// default part
		for (int i = 1; i <= nbParts; i++) {
			Part part = new Part(i, String.format("%s %d", I18N.getMsg("part"), i));
			if (objective != null) {
				part.setObjectiveChars(computeObjectiveChars(nature, nbParts));
				part.setObjectiveTime(DateUtil.addDateTimeToTS(objective, null));
			}
			session.save(part);
		}
		// first chapter
		Part defPart = new PartDAO(session).findFirst();
		for (int i = 1; i <= nbChapters; i++) {
			Chapter chapter = new Chapter(defPart, i, String.format("%s %d", I18N.getMsg("chapter"), i));
			if (objective != null) {
				chapter.setObjectiveChars(computeObjectiveChars(nature, nbChapters));
				chapter.setObjectiveTime(DateUtil.addDateTimeToTS(objective, null));
			}
			session.save(chapter);
		}
		// first scene
		Chapter chapter = new ChapterDAO(session).findFirst();
		Scene scene = EntityUtil.createScene(strand, chapter);
		session.save(scene);

		// default genders
		Gender male = new Gender(I18N.getMsg("person.gender.male"), 6, 12, 18, 65);
		session.save(male);
		Gender female = new Gender(I18N.getMsg("person.gender.female"), 6, 12, 18, 65);
		session.save(female);

		// default categories
		Category major = new Category(1, I18N.getMsg("category.central_character"), null);
		session.save(major);
		Category minor = new Category(2, I18N.getMsg("category.minor_character"), null);
		session.save(minor);

		session.getTransaction().commit();
	}

	@Override
	public synchronized void initSession(String dbName) {
		//LOG.trace("Model.initSession(" + dbName + ")");
		try {
			super.initSession(dbName);
			Session session = beginTransaction();
			// test queries
			sessionFactory.test(new PartDAO(session));
			sessionFactory.test(new ChapterDAO(session));
			commit();
			LOG.trace("SQL query test OK");
		} catch (Exception e) {
			LOG.log("*** SQL query test not OK");
		}
	}

	public void print(SbView view) {
		//App.trace("Model.print(view="+view.toString()+")");
		/*switch (SbView.getVIEW(view)) {
			case ATTRIBUTESLIST:
			case BOOK:
			case CHRONO:
			case TIMELINE:
			case INFO:
			case MANAGE:
			case NAVIGATION:
			case READING:

		// charts
			case CHART_GANTT:
			case CHART_OCCURRENCE_OF_ITEMS:
			case CHART_OCCURRENCE_OF_LOCATIONS:
			case CHART_OCCURRENCE_OF_PERSONS:
			case CHART_PERSONS_BY_DATE:
			case CHART_PERSONS_BY_SCENE:
			case CHART_STRANDS_BY_DATE:
			case CHART_WIWW:

		// others
			case MEMORIA:
			case PLAN:
			default:
				break;
		}*/
	}

	public void fireTableUpdate(AbstractEntity old, AbstractEntity entity) {
		if (old == null) {
			firePropertyChange(Book.getTYPE(entity), NEW, null, entity);
		} else if (entity == null) {
			firePropertyChange(Book.getTYPE(old), DELETE, null, entity);
		} else {
			firePropertyChange(Book.getTYPE(entity), UPDATE, old, entity);
		}
	}

	private void initProgress() {
		progressFrame = new JFrame();
		progressFrame.setUndecorated(true);
		progressFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JPanel pane = (JPanel) progressFrame.getContentPane();
		pane.setLayout(new MigLayout(MIG.WRAP));
		pane.add(new JLabel(I18N.getMsg("refresh")), MIG.CENTER);
		progressBar = new JProgressBar(0, toRefresh.length);
		progressBar.setMinimumSize(new Dimension(410, FontUtil.getFontHeight()));
		progressBar.setStringPainted(true);
		pane.add(progressBar, MIG.GROWX);
		progressFrame.pack();
		progressFrame.setLocationRelativeTo(mainFrame);
		progressFrame.setVisible(true);
	}

	Book.TYPE toRefresh[] = {
		ATTRIBUTE,
		CATEGORY,
		CHAPTER,
		ENDNOTE,
		ENDNOTE,
		EPISODE,
		EVENT,
		GENDER,
		IDEA,
		ITEM,
		ITEMLINK,
		LOCATION,
		MEMO,
		PART,
		PERSON,
		PLOT,
		RELATION,
		SCENE,
		STRAND,
		TAG,
		TAGLINK
	};

	@Override
	public void fireAgain() {
		//LOG.trace(TT + ".fireAgain()");
		initProgress();
		new Thread(new ExecThread()).start();
	}

	public void fireAgain(SbView view) {
		//LOG.trace(TT + ".fireAgain(" + view.getName() + ")");
		if (view == null || !view.isLoaded()) {
			return;
		}
		switch (SbView.getVIEW(view.getName())) {
			case BOOK:
				fireAgainScenes();
				break;
			case CHRONO:
				fireAgainScenes();
				break;
			case MANAGE:
				fireAgainChapters();
				break;
			case READING:
				fireAgainChapters();
				break;
			case TIMELINE:
				fireAgainScenes();
				break;

			case ATTRIBUTES:
				fireAgainAttributes();
				break;
			case CATEGORIES:
				fireAgainCategories();
				break;
			case CHAPTERS:
				fireAgainChapters();
				break;
			case ENDNOTES:
				fireAgainEndnotes();
				break;
			case EPISODES:
				fireAgainEpisodes();
				break;
			case EVENTS:
				fireAgainEvents();
				break;
			case GENDERS:
				fireAgainGenders();
				break;
			case IDEAS:
				fireAgainIdeas();
				break;
			case INTERNALS:
				fireAgainInternals();
			case ITEMS:
				fireAgainItems();
				break;
			case ITEMLINKS:
				fireAgainItemlinks();
				break;
			case LOCATIONS:
				fireAgainLocations();
				break;
			case MEMOS:
				fireAgainMemos();
				break;
			case PARTS:
				fireAgainParts();
				break;
			case PERSONS:
				fireAgainPersons();
				break;
			case PLOTS:
				fireAgainPlots();
				break;
			case RELATIONS:
				fireAgainRelations();
				break;
			case SCENES:
				fireAgainScenes();
				break;
			case STRANDS:
				fireAgainStrands();
				break;
			case TAGS:
				fireAgainTags();
				break;
			case TAGLINKS:
				fireAgainTaglinks();
				break;
			default:
				break;
		}
	}

	public void fireAgainReading() {
		fireAgainParts();
		fireAgainChapters();
		fireAgainScenes();
	}

	private void fireAgainAttributes() {
		//App.trace("Model.fireAgainAttributes()");
		firePropertyChange(ATTRIBUTE);
	}

	private void fireAgainCategories() {
		//App.trace("Model.fireAgainCategories()");
		firePropertyChange(CATEGORY);
	}

	public void fireAgainChapters() {
		//App.trace("Model.fireAgainChapters()");
		firePropertyChange(CHAPTER);
	}

	private void fireAgainEndnotes() {
		//App.trace("Model.fireAgainEndnotes()");
		firePropertyChange(ENDNOTE);
	}

	private void fireAgainEpisodes() {
		//App.trace("Model.fireAgainEndnotes()");
		firePropertyChange(EPISODE);
	}

	private void fireAgainEvents() {
		//App.trace("Model.fireAgainEvents()");
		firePropertyChange(EVENT);
	}

	private void fireAgainGenders() {
		//App.trace("Model.fireAgainGenders()");
		firePropertyChange(GENDER);
	}

	private void fireAgainIdeas() {
		//App.trace("Model.fireAgainIdeas()");
		firePropertyChange(IDEA);
	}

	private void fireAgainInternals() {
		//App.trace("Model.fireAgainInternals()");
		firePropertyChange(INTERNAL);
	}

	private void fireAgainItems() {
		//App.trace("Model.fireAgainItems()");
		firePropertyChange(ITEM);
	}

	private void fireAgainItemlinks() {
		//App.trace("Model.fireAgainItemlinks()");
		firePropertyChange(ITEMLINK);
	}

	private void fireAgainLocations() {
		//App.trace("Model.fireAgainLocations()");
		firePropertyChange(LOCATION);
	}

	private void fireAgainMemos() {
		//App.trace("Model.fireAgainMemos()");
		firePropertyChange(MEMO);
	}

	public void fireAgainParts() {
		//App.trace("Model.fireAgainParts()");
		firePropertyChange(PART);
	}

	private void fireAgainPersons() {
		//App.trace("Model.fireAgainPersons()");
		firePropertyChange(PERSON);
	}

	private void fireAgainPlots() {
		//App.trace("Model.fireAgainPlots()");
		firePropertyChange(PLOT);
	}

	private void fireAgainRelations() {
		//App.trace("Model.fireAgainRelations()");
		firePropertyChange(RELATION);
	}

	public void fireAgainScenes() {
		//App.trace("Model.fireAgainScenes()");
		firePropertyChange(SCENE);
	}

	private void fireAgainStrands() {
		//App.trace("Model.fireAgainStrands()");
		firePropertyChange(STRAND);
	}

	private void fireAgainTags() {
		//App.trace("Model.fireAgainTags()");
		firePropertyChange(TAG);
	}

	private void fireAgainTaglinks() {
		//App.trace("Model.fireAgainTaglinks()");
		firePropertyChange(TAGLINK);
	}

	// common
	public void setRefresh(SbView view) {
		//App.trace("Model.setRefresh(" + view.getName() + ")");
		firePropertyChange(Ctrl.PROPS.REFRESH.toString(), null, view);
		try {
			if (view.getComponentCount() == 0) {
				return;
			}
			Component comp = view.getComponent();
			if (comp instanceof ChronoPanel
					|| comp instanceof BookPanel
					|| comp instanceof Manage
					|| comp instanceof ReadingPanel) {
				// these views don't need a "fire again"
				return;
			}
			fireAgain(view);
		} catch (ArrayIndexOutOfBoundsException e) {
			// ignore
		}
	}

	public void setShowOptions(SbView view) {
		firePropertyChange(Ctrl.PROPS.SHOWOPTIONS.toString(), null, view);
	}

	public void setShowInfo(AbstractEntity entity) {
		firePropertyChange(Ctrl.PROPS.SHOWINFO.toString(), null, entity);
	}

	public void setShowInfo(H2File dbFile) {
		firePropertyChange(Ctrl.PROPS.SHOWINFO.toString(), null, dbFile);
	}

	public void setMemoriaShowEntity(AbstractEntity entity) {
		firePropertyChange(Ctrl.PROPS.SHOWINMEMORIA.toString(), null, entity);
	}

	public void setUnloadEditor() {
		firePropertyChange(Ctrl.PROPS.UNLOADEDITOR.toString(), null, null);
	}

	public void setSceneFilter(SceneStatus state) {
		firePropertyChange(Ctrl.PROPS.SCENE_FILTER_STATUS.toString(), null, state);
	}

	public void setStrandFilter(String strand) {
		firePropertyChange(Ctrl.PROPS.SCENE_FILTER_STRAND.toString(), null, strand);
	}

	public void setPrint(SbView view) {
		firePropertyChange(Ctrl.PROPS.PRINT.toString(), null, view);
	}

	public void setExport(SbView view) {
		EXIM.exporter(mainFrame, view);
	}

	// chrono view
	public void setChronoZoom(Integer val) {
		firePropertyChange(Ctrl.PROPS.CHRONO_ZOOM.toString(), null, val);
	}

	public void setChronoLayoutDirection(Boolean val) {
		firePropertyChange(Ctrl.PROPS.CHRONO_LAYOUTDIRECTION.toString(), null, val);
	}

	public void setChronoShowDateDifference(Boolean val) {
		firePropertyChange(Ctrl.PROPS.CHRONO_SHOWDATEDIFFERENCE.toString(), null, val);
	}

	public void setChronoShowEntity(AbstractEntity entity) {
		firePropertyChange(Ctrl.PROPS.CHRONO_SHOWENTITY.toString(), null, entity);
	}

	// book view
	public void setBookZoom(Integer val) {
		firePropertyChange(Ctrl.PROPS.BOOK_ZOOM.toString(), null, val);
	}

	public void setBookShowEntity(AbstractEntity entity) {
		firePropertyChange(Ctrl.PROPS.BOOK_SHOWENTITY.toString(), null, entity);
	}

	// manage view
	public void setManageZoom(Integer val) {
		firePropertyChange(Ctrl.PROPS.MANAGE_ZOOM.toString(), null, val);
	}

	public void setManageColumns(Integer val) {
		firePropertyChange(Ctrl.PROPS.MANAGE_COLUMNS.toString(), null, val);
	}

	public void setManageHideUnassigned() {
		firePropertyChange(Ctrl.PROPS.MANAGE_HIDEUNASSIGNED.toString(), null, null);
	}

	public void setManageVertical() {
		firePropertyChange(Ctrl.PROPS.MANAGE_VERTICAL.toString(), null, null);
	}

	public void setManageShowEntity(AbstractEntity entity) {
		firePropertyChange(Ctrl.PROPS.MANAGE_SHOWENTITY.toString(), null, entity);
	}

	// reading view
	public void setReadingZoom(Integer val) {
		firePropertyChange(Ctrl.PROPS.READING_LAYOUT.toString(), null, val);
	}

	public void setReadingFontSize(Integer val) {
		firePropertyChange(Ctrl.PROPS.READING_FONTSIZE.toString(), null, val);
	}

	//storyboard
	public void setStoryboardDirection() {
		firePropertyChange(Ctrl.PROPS.STORYBOARD_DIRECTION.toString(), null, null);
	}

	// memoria view
	public void setMemoriaLayout(Integer val) {
		firePropertyChange(Ctrl.PROPS.MEMORIA_LAYOUT.toString(), null, val);
	}

	// timeline view
	public void setTimelineZoom(Integer val) {
		firePropertyChange(Ctrl.PROPS.TIMELINE_ZOOM.toString(), null, val);
	}

	public void setTimelineOptions(String val) {
		firePropertyChange(Ctrl.PROPS.TIMELINE_OPTIONS.toString(), null, val);
	}

	// entity standard
	public void setEntityEdit(AbstractEntity entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setEntityUpdate(AbstractEntity entity) {
		//App.trace("Model.setEntityUpdate(entity="+entity.toString()+")");
		//TODO voir si on peut faire un code unique
		entity.setMaj();
		switch (Book.getTYPE(entity)) {
			case ATTRIBUTE:
				setATTRIBUTE_Update((Attribute) entity);
				break;
			case CATEGORY:
				setCATEGORY_Update((Category) entity);
				break;
			case CHAPTER:
				setCHAPTER_Update((Chapter) entity);
				break;
			case ENDNOTE:
				setENDNOTE_Update((Endnote) entity);
				break;
			case EPISODE:
				setEpisodeUpdate((Episode) entity);
				break;
			case EVENT:
				setEventUpdate((Event) entity);
				break;
			case GENDER:
				setGenderUpdate((Gender) entity);
				break;
			case IDEA:
				setIDEA_Update((Idea) entity);
				break;
			case INTERNAL:
				setINTERNAL_Update((Internal) entity);
				break;
			case ITEM:
				setITEM_Update((Item) entity);
				break;
			case ITEMLINK:
				setITEMLINK_Update((Itemlink) entity);
				break;
			case LOCATION:
				setLOCATION_Update((Location) entity);
				break;
			case MEMO:
				setMEMO_Update((Memo) entity);
				break;
			case PART:
				setPART_Update((Part) entity);
				break;
			case PERSON:
				setPERSON_Update((Person) entity);
				break;
			case PLOT:
				setPLOT_Update((Plot) entity);
				break;
			case RELATION:
				setRELATION_Update((Relationship) entity);
				break;
			case SCENE:
				setSCENE_Update((Scene) entity);
				break;
			case STRAND:
				setSTRAND_Update((Strand) entity);
				break;
			case TAG:
				setTAG_Update((Tag) entity);
				break;
			case TAGLINK:
				setTAGLINK_Update((Taglink) entity);
				break;
			default:
				LOG.err("updateEntity type not found: " + entity.getObjType());
				break;
		}
	}

	public synchronized void setEntityNew(AbstractEntity entity) {
		entity.setCreation();
		entity.setMaj();
		switch (Book.getTYPE(entity)) {
			case ATTRIBUTE:
				setATTRIBUTE_New((Attribute) entity);
				break;
			case CATEGORY:
				setCATEGORY_New((Category) entity);
				break;
			case CHAPTER:
				setCHAPTER_New((Chapter) entity);
				break;
			case ENDNOTE:
				setENDNOTE_New((Endnote) entity);
				break;
			case EPISODE:
				setEPISODE_New((Episode) entity);
				break;
			case EVENT:
				setEVENT_New((Event) entity);
				break;
			case GENDER:
				setGENDER_New((Gender) entity);
				break;
			case IDEA:
				setIDEA_New((Idea) entity);
				break;
			case INTERNAL:
				setINTERNAL_New((Internal) entity);
				break;
			case ITEM:
				setITEM_New((Item) entity);
				break;
			case ITEMLINK:
				setITEMLINK_New((Itemlink) entity);
				break;
			case LOCATION:
				setLOCATION_New((Location) entity);
				break;
			case MEMO:
				setMEMO_New((Memo) entity);
				break;
			case PART:
				setPART_New((Part) entity);
				break;
			case PERSON:
				setPERSON_New((Person) entity);
				break;
			case PLOT:
				setPLOT_New((Plot) entity);
				break;
			case RELATION:
				setRELATION_New((Relationship) entity);
				break;
			case SCENE:
				setSCENE_New((Scene) entity);
				break;
			case STRAND:
				setSTRAND_New((Strand) entity);
				break;
			case TAG:
				setTAG_New((Tag) entity);
				break;
			case TAGLINK:
				setTAGLINK_New((Taglink) entity);
				break;
			default:
				LOG.err("updateEntity type not found: " + entity.getObjType());
				break;
		}
	}

	public synchronized void deleteEntity(AbstractEntity entity) {
		if (entity == null) {
			return;
		}
		switch (Book.getTYPE(entity)) {
			case ATTRIBUTE:
				Model.this.setATTRIBUTE_Delete((Attribute) entity);
				break;
			case CATEGORY:
				Model.this.setCATEGORY_Delete((Category) entity);
				break;
			case CHAPTER:
				Model.this.setCHAPTER_Delete((Chapter) entity);
				break;
			case ENDNOTE:
				setENDNOTE_Delete((Endnote) entity);
				break;
			case EPISODE:
				Model.this.setEpisodeDelete((Episode) entity);
				break;
			case EVENT:
				setEVENT_Delete((Event) entity);
				break;
			case GENDER:
				setGENDER_Delete((Gender) entity);
				break;
			case IDEA:
				Model.this.setIDEA_Delete((Idea) entity);
				break;
			case INTERNAL:
				setINTERNAL_Delete((Internal) entity);
				break;
			case ITEM:
				setITEM_Delete((Item) entity);
				break;
			case ITEMLINK:
				setITEMLINK_Delete((Itemlink) entity);
				break;
			case LOCATION:
				setLOCATION_Delete((Location) entity);
				break;
			case MEMO:
				setMEMO_Delete((Memo) entity);
				break;
			case PART:
				setPART_Delete((Part) entity);
				break;
			case PERSON:
				setPERSON_Delete((Person) entity);
				break;
			case PLOT:
				setPLOT_Delete((Plot) entity);
				break;
			case RELATION:
				setRELATION_Delete((Relationship) entity);
				break;
			case SCENE:
				Model.this.setSCENE_Delete((Scene) entity);
				break;
			case STRAND:
				Model.this.setSTRAND_Delete((Strand) entity);
				break;
			case TAG:
				setTAG_Delete((Tag) entity);
				break;
			case TAGLINK:
				setTAGLINK_Delete((Taglink) entity);
				break;
			default:
				LOG.err("deleteEntity type not found: " + entity.getObjType());
				return;
		}
		mainFrame.setUpdated();
	}

	public synchronized void setEntityDelete(Book.TYPE type, ArrayList<Long> ids) {
		switch (type) {
			case ATTRIBUTE:
				Model.this.setATTRIBUTE_Delete(ids);
				break;
			case CATEGORY:
				setCATEGORY_Delete(ids);
				break;
			case CHAPTER:
				Model.this.setCHAPTER_Delete(ids);
				break;
			case ENDNOTE:
				setENDNOTE_Delete(ids);
				break;
			case EVENT:
				setEVENT_Delete(ids);
				break;
			case GENDER:
				setGENDER_Delete(ids);
				break;
			case IDEA:
				setIDEA_Delete(ids);
				break;
			case INTERNAL:
				setINTERNAL_Delete(ids);
				break;
			case ITEM:
				setITEM_Delete(ids);
				break;
			case ITEMLINK:
				setITEMLINK_Delete(ids);
				break;
			case LOCATION:
				setLOCATION_Delete(ids);
				break;
			case MEMO:
				setMEMO_Delete(ids);
				break;
			case PART:
				setPART_Delete(ids);
				break;
			case PERSON:
				setPERSON_Delete(ids);
				break;
			case PLOT:
				setPLOT_Delete(ids);
				break;
			case RELATION:
				setRELATION_Delete(ids);
				break;
			case SCENE:
				setSCENE_Delete(ids);
				break;
			case STRAND:
				setSTRAND_Delete(ids);
				break;
			case TAG:
				setTAG_Delete(ids);
				break;
			case TAGLINK:
				setTAGLINK_Delete(ids);
				break;
			default:
				LOG.err("deleteEntity type not found: " + type);
				break;
		}
	}

	// attribute
	public void setATTRIBUTE_Edit(Attribute entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setATTRIBUTE_Update(Attribute entity) {
		Session session = beginTransaction();
		AttributeDAO dao = new AttributeDAO(session);
		Attribute old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(ATTRIBUTE, UPDATE, old, entity);
	}

	public synchronized void setATTRIBUTE_New(Attribute entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(ATTRIBUTE, NEW, null, entity);
	}

	public synchronized void setATTRIBUTE_Delete(Attribute entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		// find linked entities
		PersonDAO dao = new PersonDAO(session);
		List<Person> persons = dao.findByAttribute(entity);
		commit();
		for (Person person : persons) {
			if (person.getAttributes().contains(entity)) {
				person.getAttributes().remove(entity);
				setPERSON_Update(person);
			}
		}
		// delete attribute
		session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(ATTRIBUTE, DELETE, entity, null);
	}

	public synchronized void setATTRIBUTE_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			AttributeDAO dao = new AttributeDAO(session);
			Attribute old = dao.find(id);
			commit();
			Model.this.setATTRIBUTE_Delete(old);
		}
	}

	// chapter
	public void setChapterEdit(Chapter entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setCHAPTER_Update(Chapter entity) {
		Session session = beginTransaction();
		ChapterDAO dao = new ChapterDAO(session);
		Chapter old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(CHAPTER, UPDATE, old, entity);
	}

	public synchronized void setCHAPTER_New(Chapter entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(CHAPTER, NEW, null, entity);
	}

	public synchronized void setCHAPTER_Delete(Chapter entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		// find scenes, set chapter to null
		ChapterDAO dao = new ChapterDAO(session);
		List<Scene> scenes = dao.findScenes(entity);
		commit();
		for (Scene scene : scenes) {
			scene.setChapter();
			setSCENE_Update(scene);
		}
		// delete chapter
		session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(CHAPTER, DELETE, entity, null);
	}

	public synchronized void setCHAPTER_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			ChapterDAO dao = new ChapterDAO(session);
			Chapter old = dao.find(id);
			commit();
			Model.this.setCHAPTER_Delete(old);
		}
	}

	// endnote
	public void setENDNOTE_Edit(Endnote entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setENDNOTE_Update(Endnote entity) {
		Session session = beginTransaction();
		EndnoteDAO dao = new EndnoteDAO(session);
		Endnote old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(ENDNOTE, UPDATE, old, entity);
	}

	public synchronized void setENDNOTE_New(Endnote entity) {
		//LOG.trace(TT + ".setENDNOTE_New(entity=" + entity.toString() + ")");
		Session session = beginTransaction();
		session.save(entity);
		session.getTransaction().commit();
		firePropertyChange(ENDNOTE, NEW, null, entity);
	}

	public synchronized void setENDNOTE_Delete(Endnote entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(ENDNOTE, DELETE, entity, null);
	}

	public synchronized void setENDNOTE_Delete(List<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			EndnoteDAO dao = new EndnoteDAO(session);
			Endnote old = dao.find(id);
			commit();
			Model.this.setENDNOTE_Delete(old);
		}
	}

	// episode
	// nothing to edit
	public void setEpisodeEdit(Episode entity) {
		//editEntity((AbstractEntity) entity);
	}

	// no fire property
	public synchronized void setEpisodeUpdate(Episode entity) {
		Session session = beginTransaction();
		EpisodeDAO dao = new EpisodeDAO(session);
		Episode old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		//firePropertyChange(EPISODE, UPDATE, old, entity);
	}

	// no fire property
	public synchronized void setEPISODE_New(Episode entity) {
		//LOG.trace(TT + ".setEpisodeNew(entity=" + App.traceEntity(entity) + ")");
		Session session = beginTransaction();
		session.save(entity);
		session.getTransaction().commit();
		commit();
		//firePropertyChange(EPISODE, NEW, null, entity);
	}

	// no fire property
	public synchronized void setEpisodeDelete(Episode entity) {
		//LOG.trace(TT + ".setEpisodeDelete(entity=" + App.traceEntity(entity) + ")");
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		//firePropertyChange(EPISODE, DELETE, entity, null);
	}

	// no delete multi episodes
	public synchronized void setEpisodeDelete(List<Long> ids) {
		/*for (Long id : ids) {
			Session session = beginTransaction();
			EpisodeDAOImpl dao = new EpisodeDAOImpl(session);
			Episode old = dao.find(id);
			commit();
			setEpisodeDelete(old);
		}*/
	}

	// event
	public void setEVENT_Edit(Event entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setEventUpdate(Event entity) {
		Session session = beginTransaction();
		EventDAO dao = new EventDAO(session);
		Event old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(EVENT, UPDATE, old, entity);
	}

	public synchronized void setEVENT_New(Event entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(EVENT, NEW, null, entity);
	}

	public synchronized void setEVENT_Delete(Event entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(EVENT, DELETE, entity, null);
	}

	public synchronized void setEVENT_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			EventDAO dao = new EventDAO(session);
			Event old = dao.find(id);
			commit();
			Model.this.setEVENT_Delete(old);
		}
	}

	// gender
	public void setGENDER_Edit(Gender entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setGenderUpdate(Gender gender) {
		Session session = beginTransaction();
		GenderDAO dao = new GenderDAO(session);
		Gender old = dao.find(gender.getId());
		commit();
		session = beginTransaction();
		session.update(gender);
		commit();
		firePropertyChange(GENDER, UPDATE, old, gender);
	}

	public synchronized void setGENDER_New(Gender gender) {
		Session session = beginTransaction();
		session.save(gender);
		commit();
		firePropertyChange(GENDER, NEW, null, gender);
	}

	public synchronized void setGENDER_Delete(Gender gender) {
		if (gender.getId() == null) {
			return;
		}
		// set gender of affected persons to "male"
		Session session = beginTransaction();
		GenderDAO dao = new GenderDAO(session);
		Gender male = dao.findMale();
		List<Person> persons = dao.findPersons(gender);
		commit();
		for (Person person : persons) {
			person.setGender(male);
			setPERSON_Update(person);
		}
		// delete gender
		session = beginTransaction();
		session.delete(gender);
		commit();
		firePropertyChange(GENDER, DELETE, gender, null);
	}

	public synchronized void setGENDER_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			GenderDAO dao = new GenderDAO(session);
			Gender old = dao.find(id);
			commit();
			Model.this.setGENDER_Delete(old);
		}
	}

	// part
	public void setPART_Edit(Part entity) {
		//firePropertyChange(BookController.PartProps.EDIT.toString(), null, entity);
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setPART_Update(Part entity) {
		Session session = beginTransaction();
		PartDAO dao = new PartDAO(session);
		Part old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(PART, UPDATE, old, entity);
	}

	public synchronized void setPART_New(Part entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(PART, NEW, null, entity);
	}

	public synchronized void setPART_Delete(Part entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		// delete chapters
		PartDAO dao = new PartDAO(session);
		List<Chapter> chapters = dao.findChapters(entity);
		commit();
		for (Chapter chapter : chapters) {
			Model.this.setCHAPTER_Delete(chapter);
		}
		// delete part
		session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(PART, DELETE, entity, null);
	}

	public synchronized void setPART_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			PartDAO dao = new PartDAO(session);
			Part old = dao.find(id);
			commit();
			Model.this.setPART_Delete(old);
		}
	}

	public synchronized void setPART_Change(Part entity) {
		//App.trace(("Model.setPART_Change(entity="+entity.toString()+")"));
		firePropertyChange(PART, CHANGE, null, entity);
	}

	// location
	public void setLOCATION_Edit(Location entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setLOCATION_Update(Location entity) {
		//App.trace("Model.setLocationUpdate(entity=" + entity.toString() + ")");
		Session session = beginTransaction();
		LocationDAO dao = new LocationDAO(session);
		Location old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(LOCATION, UPDATE, old, entity);
	}

	public synchronized void setLOCATION_New(Location entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(LOCATION, NEW, null, entity);
	}

	public synchronized void setLOCATION_Delete(Location entity) {
		if (entity.getId() == null) {
			return;
		}
		// delete scene links
		Session session = beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findByLocationLink(entity);
		commit();
		for (Scene scene : scenes) {
			scene.getLocations().remove(entity);
			setSCENE_Update(scene);
		}
		// delete tag / item links
		EntityUtil.deleteTagAndItemlinks(this, entity);
		// delete location
		session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(LOCATION, DELETE, entity, null);
	}

	public synchronized void setLOCATION_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			LocationDAO dao = new LocationDAO(session);
			Location old = dao.find(id);
			commit();
			Model.this.setLOCATION_Delete(old);
		}
	}

	// memo
	public void setMEMO_Edit(Memo entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setMEMO_Update(Memo entity) {
		Session session = beginTransaction();
		MemoDAO dao = new MemoDAO(session);
		Memo old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(MEMO, UPDATE, old, entity);
	}

	public synchronized void setMEMO_New(Memo entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(MEMO, NEW, null, entity);
	}

	public synchronized void setMEMO_Delete(Memo entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(MEMO, DELETE, entity, null);
	}

	public synchronized void setMEMO_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			MemoDAO dao = new MemoDAO(session);
			Memo old = dao.find(id);
			commit();
			Model.this.setMEMO_Delete(old);
		}
	}

	// person
	public void setPERSON_Edit(Person entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setPERSON_Update(Person entity) {
		Session session = beginTransaction();
		PersonDAO dao = new PersonDAO(session);
		Person old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(PERSON, UPDATE, old, entity);
	}

	public synchronized void setPERSON_New(Person entity) {
		//App.trace("Model.setPersonNew(" + entity.toString() + ")");
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(PERSON, NEW, null, entity);
	}

	public synchronized void setPERSON_Delete(Person entity) {
		if (entity.getId() == null) {
			return;
		}
		// delete scene links
		Session session = beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findByPerson(entity);
		commit();
		for (Scene scene : scenes) {
			scene.getPersons().remove(entity);
			setSCENE_Update(scene);
		}
		// delete tag / item links
		EntityUtil.deleteTagAndItemlinks(this, entity);
		// delete person
		session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(PERSON, DELETE, entity, null);
	}

	public synchronized void setPERSON_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			PersonDAO dao = new PersonDAO(session);
			Person old = dao.find(id);
			commit();
			Model.this.setPERSON_Delete(old);
		}
	}

	// plots
	public void setPLOT_Edit(Plot entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setPLOT_Update(Plot entity) {
		Session session = beginTransaction();
		PlotDAO dao = new PlotDAO(session);
		Plot old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(PLOT, UPDATE, old, entity);
	}

	public synchronized void setPLOT_New(Plot entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(PLOT, NEW, null, entity);
	}

	public synchronized void setPLOT_Delete(Plot entity) {
		if (entity.getId() == null) {
			return;
		}
		// delete plot assignments
		Session session = beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findByPlot(entity);
		commit();
		for (Scene scene : scenes) {
			scene.getPlots().remove(entity);
			setSCENE_Update(scene);
		}
		// delete plot
		session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(PLOT, DELETE, entity, null);
	}

	public synchronized void setPLOT_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			PlotDAO dao = new PlotDAO(session);
			Plot old = dao.find(id);
			commit();
			Model.this.setPLOT_Delete(old);
		}
	}

	// relation
	public void setRELATION_Edit(Relationship entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setRELATION_Update(Relationship entity) {
		Session session = beginTransaction();
		RelationDAO dao = new RelationDAO(session);
		Relationship old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(RELATION, UPDATE, old, entity);
	}

	public synchronized void setRELATION_New(Relationship entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(RELATION, NEW, null, entity);
	}

	public synchronized void setRELATION_Delete(Relationship entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(RELATION, DELETE, entity, null);
	}

	public synchronized void setRELATION_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			RelationDAO dao = new RelationDAO(session);
			Relationship old = dao.find(id);
			commit();
			Model.this.setRELATION_Delete(old);
		}
	}

	// category
	public void setCATEGORY_Edit(Category entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setCATEGORY_Update(Category category) {
		Session session = beginTransaction();
		CategoryDAO dao = new CategoryDAO(session);
		Category old = dao.find(category.getId());
		commit();
		session = beginTransaction();
		session.update(category);
		commit();
		firePropertyChange(CATEGORY, UPDATE, old, category);
	}

	public synchronized void setCATEGORY_New(Category category) {
		Session session = beginTransaction();
		session.save(category);
		commit();
		firePropertyChange(CATEGORY, NEW, null, category);
	}

	public synchronized void setCATEGORY_Delete(Category category) {
		if (category.getId() == null) {
			return;
		}
		// set category of affected persons to "minor"
		Session session = beginTransaction();
		CategoryDAO dao = new CategoryDAO(session);
		Category minor = dao.findMinor();
		List<Person> persons = dao.findPersons(category);
		commit();
		for (Person person : persons) {
			person.setCategory(minor);
			setPERSON_Update(person);
		}
		// delete category
		session = beginTransaction();
		session.delete(category);
		commit();
		firePropertyChange(CATEGORY, DELETE, category, null);
	}

	public synchronized void setCATEGORY_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			CategoryDAO dao = new CategoryDAO(session);
			Category old = dao.find(id);
			commit();
			Model.this.setCATEGORY_Delete(old);
		}
	}

	public synchronized void setCATEGORY_OrderDown(Category category) {
		firePropertyChange(CATEGORY, ORDERDOWN, null, category);
	}

	public synchronized void setCATEGORY_OrderUp(Category category) {
		firePropertyChange(CATEGORY, ORDERUP, null, category);
	}

	// strand
	public void setSTRAND_Edit(Strand entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setSTRAND_Update(Strand strand) {
		Session session = beginTransaction();
		StrandDAO dao = new StrandDAO(session);
		Strand old = dao.find(strand.getId());
		commit();
		session = beginTransaction();
		session.update(strand);
		commit();
		firePropertyChange(STRAND, UPDATE, old, strand);
	}

	public synchronized void setSTRAND_New(Strand strand) {
		Session session = beginTransaction();
		session.save(strand);
		commit();
		firePropertyChange(STRAND, NEW, null, strand);
	}

	public synchronized void setSTRAND_Delete(Strand strand) {
		if (strand.getId() == null) {
			return;
		}
		@SuppressWarnings("unchecked")
		List<Strand> list = EntityUtil.findEntities(mainFrame, STRAND);
		Strand def = null;
		for (Strand s : list) {
			if (!s.equals(strand)) {
				def = s;
				break;
			}
		}
		try {
			// delete episode strand
			Session session = beginTransaction();
			EpisodeDAO episodeDao = new EpisodeDAO(session);
			List<Episode> episodes = episodeDao.findByStrand(strand);
			commit();
			for (Episode episode : episodes) {
				setEpisodeDelete(episode);
			}
			// delete scene strands
			session = beginTransaction();
			SceneDAO sceneDao = new SceneDAO(session);
			List<Scene> scenes = sceneDao.findByStrands(strand);
			commit();
			for (Scene scene : scenes) {
				scene.getStrands().remove(strand);
				setSCENE_Update(scene);
			}
			// delete for scenes strand
			session = beginTransaction();
			StrandDAO strandDao = new StrandDAO(session);
			scenes = strandDao.findScenes(strand);
			commit();
			for (Scene scene : scenes) {
				scene.setStrand(def);
				setSCENE_Update(scene);
			}
			// delete strand
			session = beginTransaction();
			session.delete(strand);
			commit();
		} catch (ConstraintViolationException e) {
			LOG.err("Model.setDeleteStrand(" + strand.getName() + ")", e);
		}
		firePropertyChange(STRAND, DELETE, strand, null);
	}

	public synchronized void setSTRAND_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			StrandDAO dao = new StrandDAO(session);
			Strand old = dao.find(id);
			commit();
			Model.this.setSTRAND_Delete(old);
		}
	}

	public synchronized void setSTRAND_OrderUp(Strand strand) {
		firePropertyChange(STRAND, ORDERUP, null, strand);
	}

	public synchronized void setSTRAND_OrderDown(Strand strand) {
		firePropertyChange(STRAND, ORDERDOWN, null, strand);
	}

	// idea
	public void setIDEA_Edit(Idea entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setIDEA_Update(Idea idea) {
		Session session = beginTransaction();
		IdeaDAO dao = new IdeaDAO(session);
		Idea old = dao.find(idea.getId());
		commit();
		session = beginTransaction();
		session.update(idea);
		commit();
		firePropertyChange(IDEA, UPDATE, old, idea);
	}

	public synchronized void setIDEA_New(Idea idea) {
		Session session = beginTransaction();
		session.save(idea);
		commit();
		firePropertyChange(IDEA, NEW, null, idea);
	}

	public synchronized void setIDEA_Delete(Idea idea) {
		if (idea.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(idea);
		commit();
		firePropertyChange(IDEA, DELETE, idea, null);
	}

	public synchronized void setIDEA_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			IdeaDAO dao = new IdeaDAO(session);
			Idea old = dao.find(id);
			commit();
			Model.this.setIDEA_Delete(old);
		}
	}

	// tags
	public void setTAG_Edit(Tag entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setTAG_Update(Tag tag) {
		Session session = beginTransaction();
		TagDAO dao = new TagDAO(session);
		Tag old = dao.find(tag.getId());
		commit();
		session = beginTransaction();
		session.update(tag);
		commit();
		firePropertyChange(TAG, UPDATE, old, tag);
	}

	public synchronized void setTAG_New(Tag tag) {
		Session session = beginTransaction();
		session.save(tag);
		commit();
		firePropertyChange(TAG, NEW, null, tag);
	}

	public synchronized void setTAG_Delete(Tag tag) {
		if (tag.getId() == null) {
			return;
		}
		// delete tag assignments
		Session session = beginTransaction();
		TaglinkDAO dao = new TaglinkDAO(session);
		List<Taglink> links = dao.findByTag(tag);
		commit();
		for (Taglink link : links) {
			Model.this.setTAGLINK_Delete(link);
		}
		// delete tag
		session = beginTransaction();
		session.delete(tag);
		commit();
		firePropertyChange(TAG, DELETE, tag, null);
	}

	public synchronized void setTAG_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			TagDAO dao = new TagDAO(session);
			Tag old = dao.find(id);
			commit();
			Model.this.setTAG_Delete(old);
		}
	}

	// items
	public void setITEM_Edit(Item entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setITEM_Update(Item entity) {
		Session session = beginTransaction();
		ItemDAO dao = new ItemDAO(session);
		Item old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(ITEM, UPDATE, old, entity);
	}

	public synchronized void setITEM_New(Item entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(ITEM, NEW, null, entity);
	}

	public synchronized void setITEM_Delete(Item item) {
		if (item.getId() == null) {
			return;
		}
		// delete item assignments
		Session session = beginTransaction();
		ItemlinkDAO dao = new ItemlinkDAO(session);
		List<Itemlink> links = dao.findByItem(item);
		commit();
		for (Itemlink link : links) {
			Model.this.setITEMLINK_Delete(link);
		}
		// delete item
		session = beginTransaction();
		session.delete(item);
		commit();
		firePropertyChange(ITEM, DELETE, item, null);
	}

	public synchronized void setITEM_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			ItemDAO dao = new ItemDAO(session);
			Item old = dao.find(id);
			commit();
			Model.this.setITEM_Delete(old);
		}
	}

	// tag links
	public void setTAGLINK_Edit(Taglink entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setTAGLINK_Update(Taglink entity) {
		Session session = beginTransaction();
		TaglinkDAO dao = new TaglinkDAO(session);
		Taglink old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(TAGLINK, UPDATE, old, entity);
	}

	public synchronized void setTAGLINK_New(Taglink entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(TAGLINK, NEW, null, entity);
	}

	public synchronized void setTAGLINK_Delete(Taglink entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(TAGLINK, DELETE, entity, null);
	}

	public synchronized void setTAGLINK_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			TaglinkDAO dao = new TaglinkDAO(session);
			Taglink old = dao.find(id);
			commit();
			Model.this.setTAGLINK_Delete(old);
		}
	}

	// item links
	public void setITEMLINK_Edit(Itemlink entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setITEMLINK_Update(Itemlink entity) {
		Session session = beginTransaction();
		ItemlinkDAO dao = new ItemlinkDAO(session);
		Itemlink old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(ITEMLINK, UPDATE, old, entity);
	}

	public synchronized void setITEMLINK_New(Itemlink entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(ITEMLINK, NEW, null, entity);
	}

	public synchronized void setITEMLINK_Delete(Itemlink entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(ITEMLINK, DELETE, entity, null);
	}

	public synchronized void setITEMLINK_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			ItemlinkDAO dao = new ItemlinkDAO(session);
			Itemlink old = dao.find(id);
			commit();
			Model.this.setITEMLINK_Delete(old);
		}
	}

	// scenes
	public void setSCENE_Edit(Scene entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setSCENE_Update(Scene entity) {
		//App.trace("Model.setSceneUpdate(entity=" + entity.toString() + ")");
		Session session = beginTransaction();
		Scene old = (Scene) session.get(Scene.class, entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(SCENE, UPDATE, old, entity);
	}

	public synchronized void setSCENE_New(Scene entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		firePropertyChange(SCENE, NEW, null, entity);
	}

	public synchronized void setSCENE_Delete(Scene entity) {
		if (entity.getId() == null) {
			return;
		}
		// delete tag / item links
		EntityUtil.deleteTagAndItemlinks(this, entity);
		// remove relative scene of affected scenes
		Session session = beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findScenesWithRelativeSceneId(entity);
		commit();
		for (Scene scene2 : scenes) {
			scene2.removeRelativescene();
			setSCENE_Update(scene2);
		}
		// delete scene
		session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(SCENE, DELETE, entity, null);
	}

	public synchronized void setSCENE_Delete(List<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			SceneDAO dao = new SceneDAO(session);
			Scene old = dao.find(id);
			commit();
			Model.this.setSCENE_Delete(old);
		}
	}

	// internals
	public void setINTERNAL_Edit(Internal entity) {
		editEntity((AbstractEntity) entity);
	}

	public synchronized void setINTERNAL_Update(Internal entity) {
		Session session = beginTransaction();
		InternalDAO dao = new InternalDAO(session);
		Internal old = dao.find(entity.getId());
		commit();
		session = beginTransaction();
		session.update(entity);
		commit();
		firePropertyChange(INTERNAL, UPDATE, old, entity);
	}

	public synchronized void setINTERNAL_New(Internal entity) {
		Session session = beginTransaction();
		session.save(entity);
		commit();
		String prop = new ActKey(Book.TYPE.INTERNAL, Ctrl.PROPS.NEW).toString();
		firePropertyChange(prop, null, entity);
	}

	public synchronized void setINTERNAL_Delete(Internal entity) {
		if (entity.getId() == null) {
			return;
		}
		Session session = beginTransaction();
		session.delete(entity);
		commit();
		firePropertyChange(INTERNAL, DELETE, entity, null);
	}

	public synchronized void setINTERNAL_Delete(ArrayList<Long> ids) {
		for (Long id : ids) {
			Session session = beginTransaction();
			InternalDAO dao = new InternalDAO(session);
			Internal old = dao.find(id);
			commit();
			Model.this.setINTERNAL_Delete(old);
		}
	}

	public void refreshParts() {
		SbView v = mainFrame.getView(VIEWNAME.PARTS);
		if (v != null) {
			fireAgain(v);
		}
	}

	public void refreshChapters() {
		SbView v = mainFrame.getView(VIEWNAME.CHAPTERS);
		if (v != null) {
			fireAgain(v);
		}
	}

	public void refreshScenes() {
		SbView v = mainFrame.getView(VIEWNAME.SCENES);
		if (v != null) {
			fireAgain(v);
		}
	}

	public class ExecThread implements Runnable {

		public ExecThread() {
		}

		@Override
		public void run() {
			int i = 0, total = toRefresh.length;
			for (Book.TYPE type : toRefresh) {
				progressBar.setValue(i);
				firePropertyChange(type);
				i++;
				progressBar.setString(i + "/" + total
						+ " (" + I18N.getMsg(type.toString()) + ")");
			}
			progressFrame.dispose();
		}
	}

}
