/*
Copyright 2018 by FaVdB
Modifications ???? by ????

This file is part of oStorybook.

    oStorybook is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    oStorybook is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with oStorybook.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.book;

import java.util.ArrayList;
import java.util.List;
import storybook.tools.ListUtil;
import storybook.tools.html.Html;

/**
 *
 * @author FaVdB
 */
public class BookParamExport extends AbstractParam {

	public enum FORMAT {
		XML,
		HTML,
		OSBD,
		CSV,
		TXT,
		NONE;

		@Override
		public String toString() {
			return this.name().toLowerCase();
		}

		public boolean isHtml() {
			return toString().equals("html");
		}

		public boolean isXml() {
			return toString().equals("xml");
		}

		public boolean isOsbd() {
			return toString().equals("osbd");
		}

		public boolean compare(String str) {
			return this.name().toLowerCase().equals(str.toLowerCase());
		}
	}

	public static FORMAT getFormat(String str) {
		if (str != null && !str.isEmpty()) {
			for (FORMAT id : FORMAT.values()) {
				if (id.toString().equals(str)) {
					return (id);
				}
			}
		}
		return (FORMAT.NONE);
	}

	private String directory;
	private String format;
	private boolean chapterBookTitle;
	private boolean chapterBreakPage;
	private String htmlCss;
	private boolean htmlNav;
	private boolean htmlNavImage;
	private boolean htmlMultiChapter;
	private boolean htmlMultiScene;
	private int highlight = Html.EM_LEFTASIS;
	private String fileName;
	private String csvQuote = "\"";
	private String csvComma = ";";
	private boolean txtTab;
	private String txtSeparator;

	public BookParamExport(BookParam param) {
		super(param, "export");
		init();
	}

	public String getDirectory() {
		return (directory);
	}

	public void setDirectory(String d) {
		directory = d;
	}

	public String getFormat() {
		return format;
	}

	public void setFormat(String d) {
		format = d;
	}

	public boolean getChapterBookTitle() {
		return (chapterBookTitle);
	}

	public void setChapterBookTitle(boolean d) {
		chapterBookTitle = d;
	}

	public boolean getChapterBreakPage() {
		return (chapterBreakPage);
	}

	public void setChapterBreakPage(boolean d) {
		chapterBreakPage = d;
	}

	public void setHighlight(int value) {
		this.highlight = value;
	}

	public int getHighlight() {
		return this.highlight;
	}

	public String getHtmlCss() {
		return (htmlCss);
	}

	public void setHtmlCss(String d) {
		htmlCss = d;
	}

	public boolean isMulti() {
		return htmlMultiChapter || htmlMultiScene;
	}

	public boolean getHtmlMultiChapter() {
		return (htmlMultiChapter);
	}

	public void setHtmlMultiChapter(boolean d) {
		htmlMultiChapter = d;
	}

	public boolean getHtmlMultiScene() {
		return (htmlMultiScene);
	}

	public void setHtmlMultiScene(boolean d) {
		htmlMultiScene = d;
	}

	public boolean getHtmlNav() {
		return (htmlNav);
	}

	public void setHtmlNav(boolean d) {
		htmlNav = d;
	}

	public boolean getHtmlNavImage() {
		return (htmlNavImage);
	}

	public void setHtmlNavImage(boolean d) {
		htmlNavImage = d;
	}

	@Override
	protected void init() {
		Book pBook = param.book;
		setDirectory(pBook.getString(Book.PARAM.EXPORT_DIR));
		setChapterBookTitle(pBook.getBoolean(Book.PARAM.EXPORT_CHAPTER_BOOKTITLE));
		setChapterBreakPage(pBook.getBoolean(Book.PARAM.EXPORT_CHAPTER_BREAKPAGE));
		setHtmlCss(pBook.getString(Book.PARAM.EXPORT_HTML_CSS));
		setHtmlNav(pBook.getBoolean(Book.PARAM.EXPORT_HTML_NAV));
		setHtmlNavImage(pBook.getBoolean(Book.PARAM.EXPORT_HTML_NAVIMAGE));
		setHtmlMultiChapter(pBook.getBoolean(Book.PARAM.EXPORT_HTML_MULTICHAPTER));
		setHtmlMultiScene(pBook.getBoolean(Book.PARAM.EXPORT_HTML_MULTISCENE));
		setFormat(pBook.getString(Book.PARAM.EXPORT_FORMAT));
		setCsvQuote(pBook.getString(Book.PARAM.EXPORT_CSV_QUOTE));
		setCsvComma(pBook.getString(Book.PARAM.EXPORT_CSV_COMMA));
		setTxtTab(pBook.getBoolean(Book.PARAM.EXPORT_TXT_TAB));
		setTxtSeparator(pBook.getString(Book.PARAM.EXPORT_TXT_SEPARATOR));
	}

	@Override
	public void save() {
		Book pBook = param.book;
		pBook.setString(Book.PARAM.EXPORT_DIR, directory);
		pBook.setBoolean(Book.PARAM.EXPORT_CHAPTER_BOOKTITLE, chapterBookTitle);
		pBook.setBoolean(Book.PARAM.EXPORT_CHAPTER_BREAKPAGE, chapterBreakPage);
		pBook.setString(Book.PARAM.EXPORT_HTML_CSS, htmlCss);
		pBook.setBoolean(Book.PARAM.EXPORT_HTML_NAV, htmlNav);
		pBook.setBoolean(Book.PARAM.EXPORT_HTML_NAVIMAGE, htmlNavImage);
		pBook.setBoolean(Book.PARAM.EXPORT_HTML_MULTICHAPTER, htmlMultiChapter);
		pBook.setBoolean(Book.PARAM.EXPORT_HTML_MULTISCENE, htmlMultiScene);
		pBook.setString(Book.PARAM.EXPORT_FORMAT, format);
		pBook.setString(Book.PARAM.EXPORT_CSV_QUOTE, csvQuote);
		pBook.setString(Book.PARAM.EXPORT_CSV_COMMA, csvComma);
		pBook.setBoolean(Book.PARAM.EXPORT_TXT_TAB, txtTab);
		pBook.setString(Book.PARAM.EXPORT_TXT_SEPARATOR, txtSeparator);
	}

	public String toTrace() {
		StringBuilder b = new StringBuilder();
		b.append("export dir=").append(param.getParamExport().directory);
		List<String> str = new ArrayList<>();
		if (param.getParamExport().chapterBookTitle) {
			str.add("book title");
		}
		if (param.getParamExport().isMulti()) {
			str.add("multiple files");
		}
		if (param.getParamExport().htmlMultiChapter) {
			str.add("chapters");
		}
		if (param.getParamExport().htmlMultiScene) {
			str.add("scenes");
		}
		if (param.getParamExport().highlight == 0) {
			str.add("ignore highlighter");
		}
		if (param.getParamExport().highlight == 1) {
			str.add("remove highlighter");
		}
		if (param.getParamExport().highlight == 2) {
			str.add("highlighter as is");
		}
		if (param.getParamExport().htmlNav) {
			str.add("nav");
		}
		if (param.getParamExport().htmlNavImage) {
			str.add("with images");
		}
		if (param.getParamExport().chapterBreakPage) {
			str.add("chapter break page");
		}
		b.append("\nparameters=").append(ListUtil.join(str));
		b.append("\ncss=").append(param.getParamExport().htmlCss);
		b.append("format=").append(param.getParamExport().format);
		return b.toString();
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}

	public String getFileName() {
		return fileName;
	}

	public boolean isHtml() {
		return format.equals("html");
	}

	public boolean isTxt() {
		return format.equals("txt");
	}

	public boolean isXml() {
		return format.equals("xml");
	}

	public boolean isOsbk() {
		return format.equals("osbk");
	}

	public String getTxtSeparator() {
		return this.txtSeparator;
	}

	public void setTxtSeparator(String value) {
		this.txtSeparator = value;
	}

	public boolean isTxtTab() {
		return txtTab;
	}

	public void setTxtTab(boolean sel) {
		txtTab = sel;
	}

	public void setCsvQuote(String value) {
		this.csvQuote = value;
	}

	public String getCsvQuote() {
		return this.csvQuote;
	}

	public void setCsvComma(String value) {
		this.csvComma = value;
	}

	public String getCsvComma() {
		return this.csvComma;
	}

	public BookParamLayout getLayout() {
		return this.param.getParamLayout();
	}

	public String toCheck() {
		StringBuilder b = new StringBuilder();
		b.append("format=").append(format).append("|");
		b.append("csvComma=").append(csvComma).append("|");
		b.append("csvQuote=").append(csvQuote).append("|");
		b.append("dir=").append(directory).append("|");
		b.append("fileName=").append(fileName).append("|");
		b.append("txtSeparator=").append(txtSeparator).append("|");
		b.append("txtTab=").append(txtTab ? "1" : "0");
		return b.toString();
	}

}
