/*
 Storybook: Open Source software for novelists and authors.
 Copyright (C) 2008 - 2012 Martin Mustun

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.entity;

import i18n.I18N;
import java.awt.Color;
import java.io.File;
import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.Icon;
import org.w3c.dom.Node;
import storybook.edit.Intensity;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import static storybook.model.hbn.entity.AbstractEntity.getXmlText;
import storybook.model.hbn.entity.DB_DATA.DATA;
import storybook.model.state.SceneStatus;
import storybook.model.state.SceneStatusModel;
import storybook.tools.DateUtil;
import storybook.tools.SbDuration;
import storybook.tools.TextUtil;
import storybook.tools.html.Html;
import storybook.tools.swing.ColorUtil;
import storybook.tools.zip.ZipXml;
import storybook.ui.MainFrame;

/**
 * Scene generated by hbm2java
 *
 * @hibernate.class table="SCENE"
 */
public class Scene extends AbstractEntity {

	private static final String STD_FMT = "%s.%02d";

	private Chapter chapter;
	private Strand strand;
	private Integer sceneno = 0;
	private Timestamp scenets;
	//TODO calendar
	//private String sbDate;
	private String title = "";
	private String summary = "";
	private Integer status = 0;// means DUMMY status
	private Integer intensity = 1;
	private String duration = "";
	private Integer relativeDateDifference = 0;
	private String relativetime = "";
	private Long relativesceneid = -1L;
	private String odf = "";
	private Boolean informative = false;

	private List<Person> persons = new ArrayList<>();
	private List<Item> items = new ArrayList<>();
	private List<Location> locations = new ArrayList<>();
	private List<Plot> plots = new ArrayList<>();
	private List<Strand> strands = new ArrayList<>();
	private Person narrator = null;

	//scenario
	public String scenario_pitch = "";
	public Integer scenario_moment = 0,
			//0=NONE, 1=DAY, 2=AT DAWN, 3=MORNING, 4=MIDDAY,
			//5=AFTERNOON, 6=SUNSET, 7=NIGHT, 8=EVENING, 9=LATER
			scenario_loc = 0, //0=NONE,1=INT.,2=EXT.
			scenario_start = 0, //0=NONE,1=CUT,2=FADE IN
			scenario_end = 0, //0=NONE,1=CUT,2=FADE OUT,3=DISSOLVE TO
			scenario_stage = 0 //0=none, else the index number of the stage
			;

	public Scene() {
		super(Book.TYPE.SCENE, "111");
		//setDate(new Date());
		setSummary("");
		setNotes("");
		setDescription("");
		setAssistant("");
		scenario_pitch = "";
		scenario_moment = 0;
		scenario_loc = 0;
		scenario_start = 0;
		scenario_end = 0;
		scenario_stage = 0;
	}

	public Scene(MainFrame mainFrame) {
		this();
		if (mainFrame != null) {
			switch (mainFrame.getBook().getInteger(Book.PARAM.SCENE_DATE_INIT, 1)) {
				case 0:
					setDate(null);
					break;
				case 1:
					setDate(mainFrame.sceneGetLastDate());
					break;
				case 2:
					setDate(new Date());
					break;
			}
		}
	}

	public Scene(Integer number, String title, Chapter chapter, Strand strand) {
		this();
		setName(title);
		setChapter(chapter);
		setStrand(strand);
		setSceneno(number);
		setStatus(1);
	}

	public Scene(Chapter chapter, Strand strand, Integer number, Timestamp date,
			String title, String summary, Integer status, String duration,
			Integer relativeDateDifference, Long relativesceneid,
			String notes, Boolean informative, String odf) {
		this();
		setName(title);
		setChapter(chapter);
		setStrand(strand);
		setSceneno(number);
		setScenets(date);
		setTitle(title);
		setSummary(summary);
		setStatus(status);
		setRelativeDateDifference(relativeDateDifference);
		setRelativesceneid(relativesceneid);
		setDuration(duration);
		setNotes(notes);
		setInformative(informative);
		setOdf(odf);
	}

	public Chapter getChapter() {
		return this.chapter;
	}

	public boolean hasChapter() {
		return chapter != null;
	}

	public void setChapter(Chapter chapter) {
		this.chapter = chapter;
	}

	public void setChapter() {
		this.chapter = null;
	}

	public Strand getStrand() {
		return this.strand;
	}

	public void setStrand() {
		this.strand = null;
	}

	public void setStrand(Strand strand) {
		this.strand = strand;
	}

	@SuppressWarnings("null")
	public Integer getSceneno() {
		return (sceneno != null ? sceneno : -1);
	}

	public static Integer getNextNumber(List scenes) {
		int n = 0;
		for (Object c : scenes) {
			if (((Scene) c).sceneno > n) {
				n = ((Scene) c).sceneno;
			}
		}
		return (n + 1);
	}

	public boolean hasSceneno() {
		return sceneno != null;
	}

	public void setSceneno(Integer sceneno) {
		this.sceneno = sceneno;
	}

	public Timestamp getScenets() {
		return this.scenets;
	}

	public boolean hasScenets() {
		return scenets != null;
	}

	public int getDateType() {
		if (hasDate()) {
			return 1;
		}
		if (hasRelativescene()) {
			return 2;
		}
		return 0;
	}

	public Date getDate() {
		if (scenets == null) {
			return null;
		}
		return new Date(scenets.getTime());
	}

	/**
	 * return true if scene has a TS or if has a relative scene
	 *
	 * @return
	 */
	@Override
	public boolean hasDate() {
		return hasScenets() || hasRelativescene();
	}

	public void removeScenets() {
		scenets = null;
	}

	public String getDateOnly() {
		if (!hasScenets()) {
			return "";
		}
		DateFormat formatter = new SimpleDateFormat("yyyy/MM/dd");
		return formatter.format(scenets);
	}

	public String getDateStrShort() {
		if (!hasScenets()) {
			return "";
		}
		DateFormat formatter;
		if (DateUtil.isZeroTimeDate(scenets)) {
			formatter = I18N.getShortDateFormatter();
		} else {
			formatter = I18N.getDateTimeFormatter();
		}
		return formatter.format(scenets);
	}

	public String getDateStrMedium() {
		if (!hasScenets()) {
			return "";
		}
		DateFormat formatter;
		if (DateUtil.isZeroTimeDate(scenets)) {
			formatter = I18N.getMediumDateFormatter();
		} else {
			formatter = I18N.getDateTimeFormatter();
		}
		return formatter.format(scenets);
	}

	public String getDateStrLong() {
		if (!hasScenets()) {
			return "";
		}
		DateFormat formatter;
		if (DateUtil.isZeroTimeDate(scenets)) {
			formatter = I18N.getLongDateFormatter();
		} else {
			formatter = I18N.getDateTimeFormatter();
		}
		return formatter.format(scenets);
	}

	public void setDate(Date date) {
		if (date == null) {
			setScenets(null);
		} else {
			setScenets(new Timestamp(date.getTime()));
		}
	}

	public void setScenets(Timestamp ts) {
		scenets = ts;
	}

	public String getTitle() {
		if (title == null) {
			title = "";
		}
		return title;
	}

	public String getTitle(boolean truncate) {
		if (truncate) {
			return TextUtil.ellipsize(title, 30);
		}
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
		setName(title);
	}

	public String getFullTitle() {
		return getFullTitle(false);
	}

	public String getFullTitle(boolean truncate) {
		return getFullTitle(false, truncate);
	}

	public String getFullTitle(boolean timestamp, boolean truncate) {
		StringBuilder buf = new StringBuilder();
		buf.append(getChapterSceneNo());
		buf.append(getTitle(truncate));
		if (timestamp && hasScenets()) {
			buf.append(" (");
			buf.append(getDateStrMedium());
			buf.append(")");
		}
		return buf.toString();
	}

	public String getTitleText(boolean truncate, int length) {
		if (title == null || title.isEmpty()) {
			return getText(length);
		}
		return getName() + ": " + getText(length);
	}

	public String getChapterSceneNo() {
		return getChapterSceneNo(true);
	}

	public String getChapterSceneNo(boolean appendColon) {
		StringBuilder buf = new StringBuilder();
		if (hasChapter()) {
			buf.append(chapter.getChapterno());
		} else {
			buf.append("x");
		}
		buf.append(".");
		if (hasSceneno()) {
			buf.append(sceneno);
		} else {
			buf.append("x");
		}
		if (appendColon) {
			buf.append(": ");
		}
		return buf.toString();
	}

	public String getChapterSceneToolTip() {
		if (!hasChapter()) {
			return "";
		}
		StringBuilder buf = new StringBuilder("<html>");
		buf.append(I18N.getColonMsg("chapter"));
		buf.append(" ").append(getChapter().toString());
		buf.append("<br>");
		if (getChapter().hasPart()) {
			buf.append(I18N.getColonMsg("part"));
			buf.append(" ").append(getChapter().getPart().toString());
			buf.append("<br>");
		}
		return buf.toString();
	}

	public String getPartChapterSceneNo() {
		String ret;
		if (!hasChapter()) {
			ret = "x.x.";
		} else {
			ret = chapter.getPart().getNumber() + "." + chapter.getChapterno() + ".";
		}
		ret += sceneno;
		return (ret);
	}

	public boolean hasEndnote(String note) {
		return (getSummary().contains(note));
	}

	public String getSummary() {
		if (summary == null) {
			return ("<p></p>");
		}
		// suppression de codes HTML
		String toRemove[] = {"html", "div", "body"};
		String text = this.summary;
		if (text.contains("<head.*</head>")) {
			text = text.replaceAll("<head.*</head>", "");
		}
		for (String str : toRemove) {
			text = text.replaceAll("<" + str + ".*>", "");
			text = text.replaceAll("</" + str + ">", "");
		}
		if (text.isEmpty()) {
			text = "<p></p>";
		}
		return (text);
	}

	public String getSummary(int length) {
		return TextUtil.ellipsize(Html.htmlToText(summary), length);
	}

	public int getChars() {
		int nb = 0;
		if (getOdf() != null && !getOdf().isEmpty()) {
			File f = new File(getOdf());
			if (f.exists()) {
				if (getOdf().endsWith("odt")) {
					nb = TextUtil.countChars(ZipXml.getText(f, "content.xml", "text:h, text:p"));
				} else if (getOdf().endsWith("docx")) {
					nb = TextUtil.countChars(ZipXml.getText(f, "word/document.xml", "w:t"));
				} else {
					nb = TextUtil.countChars(getSummary());
				}
				return nb;
			}
		}
		return TextUtil.countChars(getSummary());
	}

	public static int getChars(List<Scene> scenes) {
		int n = 0;
		for (Scene s : scenes) {
			n += s.getChars();
		}
		return (n);
	}

	public String getCCSS() {
		int cn = 0;
		if (hasChapter()) {
			cn = getChapter().getChapterno();
		}
		String str = String.format("%02d.%02d", cn, getSceneno());
		return str;
	}

	public int getWords() {
		int r = 0;
		if (getOdf() != null && !getOdf().isEmpty()) {
			File f = new File(getOdf());
			if (f.exists()) {
				return TextUtil.countWords(ZipXml.getText(f));
			}
		}
		return TextUtil.countWords(getSummary());
	}

	public static int getWords(List<Scene> scenes) {
		int n = 0;
		for (Scene s : scenes) {
			n += s.getWords();
		}
		return (n);
	}

	public static int getIntensity(List<Scene> scenes) {
		int n = 0;
		for (Scene s : scenes) {
			n += s.getIntensity();
		}
		if (n == 0 || scenes.size() < 1) {
			return 1;
		}
		return Math.max(1, n / scenes.size());
	}

	public String getText() {
		return getSummary();
	}

	public String getTextOnly() {
		return getSummary();
	}

	public String getText(int length) {
		return getSummary(length);
	}

	public String getTextToHtml(boolean isMulti) {
		String txt = getSummary();
		if (isMulti) {
			txt = txt.replace("#endnote", "./endnote.html#endnote");
		}
		return txt;
	}

	public void setSummary(String str) {
		if (str == null) {
			this.summary = null;
		} else {
			this.summary = str
					.replace("<html>", "").replace("</html>", "")
					.replace("<head>", "").replace("</head>", "")
					.replace("<body>", "").replace("</body>", "")
					.replaceAll("<style.*type=\"text/css.*</style>", "");
		}
	}

	public Integer getStatus() {
		if (status == null) {
			return 0;
		}
		if (status >= Status.STATUS.values().length) {
			status = 0;
		}
		return status;
	}

	public Icon getStatusIcon() {
		return Status.getStatusIcon(getStatus());
	}

	public void setStatus(Integer status) {
		//App.trace("scene setting status to " + status);
		this.status = status;
	}

	public void setScenestate(SceneStatus state) {
		this.status = state.getNumber();
	}

	public SceneStatus getScenestate() {
		if (status == null) {
			status = 0;
		}
		SceneStatusModel model = new SceneStatusModel();
		return (SceneStatus) model.findByNumber(this.status);
	}

	public Integer getIntensity() {
		if (intensity == null) {
			return 1;
		}
		if (intensity >= 5) {
			intensity = 5;
		}
		return this.intensity;
	}

	public Color getIntensityColor() {
		return Intensity.getColorValue(intensity);
	}

	public void setIntensity(Integer intensity) {
		this.intensity = intensity;
	}

	public void setDuration(String duration) {
		this.duration = duration;
	}

	public String getDuration() {
		return (duration == null ? "" : duration);
	}

	public boolean hasDuration() {
		return (duration != null && !duration.isEmpty());

	}

	public SbDuration getSbDuration() {
		if (hasDuration()) {
			return new SbDuration(getDuration());
		} else {
			return new SbDuration(SbDuration.computeFromWords(summary));
		}
	}

	public String getDurationToText() {
		if (!getDuration().isEmpty()) {
			SbDuration du = new SbDuration(duration);
			return du.toText();
		}
		int words = getWords();
		if (words == 0) {
			return "";
		}
		SbDuration d = new SbDuration(SbDuration.computeFromWords(summary));
		String r = d.toText();
		return "[" + d.toText() + "]";
	}

	public String getTextDuration() {
		if ((duration == null || duration.isEmpty()) && !summary.isEmpty()) {
			return "[" + SbDuration.computeFromWords(summary) + "]";
		}
		return getDuration();
	}

	public Integer getRelativeDateDifference() {
		return this.relativeDateDifference;
	}

	public void setRelativeDateDifference(Integer relativeDateDifference) {
		this.relativeDateDifference = relativeDateDifference;
	}

	public String getRelativetime() {
		return (this.relativetime == null ? "" : this.relativetime);
	}

	public void setRelativetime(String rel) {
		this.relativetime = rel;
	}

	public Date getRelativedate(Scene relativeScene) {
		if (relativeScene == null) {
			return null;
		}
		Date date = relativeScene.getDate();
		if ((date == null) || (this.relativetime == null || this.relativetime.isEmpty())) {
			return null;
		}
		return DateUtil.add(date, relativetime);
	}

	public Long getRelativesceneid() {
		if (this.relativesceneid == null) {
			return -1L;
		}
		return this.relativesceneid;
	}

	public void setRelativesceneid(Long relativeSceneId) {
		this.relativesceneid = relativeSceneId;
	}

	public void setRelativesceneid(Scene relativeScene) {
		if (relativeScene == null) {
			return;
		}
		setRelativesceneid(relativeScene.id);
	}

	public boolean hasRelativescene() {
		return this.relativesceneid != -1L;
	}

	public void removeRelativescene() {
		relativesceneid = null;
		relativetime = "";
		relativeDateDifference = null;
	}

	public List<Person> getPersons() {
		if (persons == null) {
			return (new ArrayList<>());
		}
		return persons;
	}

	@SuppressWarnings("unchecked")
	public String getPersonList() {
		return (EntityUtil.getNames((List) persons));
	}

	public void setPersons(List<Person> persons) {
		this.persons = persons;
	}

	public List<Item> getItems() {
		if (items == null) {
			return (new ArrayList<>());
		}
		return items;
	}

	public void setItems(List<Item> items) {
		this.items = items;
	}

	public List<Location> getLocations() {
		if (locations == null) {
			return (new ArrayList<>());
		}
		return locations;
	}

	@SuppressWarnings("unchecked")
	public String getLocationList() {
		return (EntityUtil.getNames((List) locations));
	}

	public void setLocations(List<Location> locations) {
		this.locations = locations;
	}

	public List<Strand> getStrands() {
		if (strands == null) {
			return (new ArrayList<>());
		}
		return strands;
	}

	@SuppressWarnings("unchecked")
	public String getStrandsList() {
		return (EntityUtil.getNames((List) strands));
	}

	public void setStrands(List<Strand> strands) {
		this.strands = strands;
	}

	public Boolean getInformative() {
		if (informative == null) {
			return false;
		}
		return informative;
	}

	public void setInformative(Boolean informative) {
		this.informative = informative;
	}

	public Person getNarrator() {
		return narrator;
	}

	public void setNarrator(Person narrator) {
		this.narrator = narrator;
	}

	public void setScenario_stage(Integer stage) {
		this.scenario_stage = stage;
	}

	public Integer getScenario_stage() {
		if (scenario_stage == null) {
			scenario_stage = 0;
		}
		return (this.scenario_stage);
	}

	public void setScenario_pitch(String pitch) {
		this.scenario_pitch = pitch;
	}

	public String getScenario_pitch() {
		return (scenario_pitch == null ? "" : scenario_pitch);
	}

	public void setScenario_moment(Integer moment) {
		this.scenario_moment = moment;
	}

	public Integer getScenario_moment() {
		if (scenario_moment == null) {
			scenario_moment = 0;
		}
		return (scenario_moment);
	}

	public void setScenario_loc(Integer loc) {
		this.scenario_loc = loc;
	}

	public Integer getScenario_loc() {
		if (scenario_loc == null) {
			scenario_loc = 0;
		}
		return (scenario_loc);
	}

	public void setScenario_start(Integer start) {
		this.scenario_start = start;
	}

	public Integer getScenario_start() {
		if (scenario_start == null) {
			scenario_start = 0;
		}
		return (scenario_start);
	}

	public void setScenario_end(Integer end) {
		this.scenario_end = end;
	}

	public Integer getScenario_end() {
		if (scenario_end == null) {
			scenario_end = 0;
		}
		return (scenario_end);
	}

	@Override
	public String toString() {
		if (isTransient()) {
			return "";
		}
		return (getFullTitle(false, true) + (this.hasNotes() ? "*" : ""));
	}

	@Override
	public String toCsv(String quoteStart, String quoteEnd, String separator) {
		StringBuilder b = new StringBuilder();
		b.append(quoteStart).append(getClean(this)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getName())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getChapter())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getStrand())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getSceneno())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getScenets())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getStatus())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getInformative())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getDuration())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getIntensity())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getRelativetime())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getRelativesceneid())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getNarrator()).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getOdf())).append(quoteEnd).append(separator);
		b.append(quoteStart);
		for (Person p : getPersons()) {
			b.append(p.getId().toString()).append("/");
		}
		b.append(quoteEnd).append(separator);
		b.append(quoteStart);
		for (Item p : getItems()) {
			b.append(p.getId().toString()).append("/");
		}
		b.append(quoteEnd).append(separator);
		b.append(quoteStart);
		for (Location p : getLocations()) {
			b.append(p.getId().toString()).append("/");
		}
		b.append(quoteEnd).append(separator);
		b.append(quoteStart);
		for (Strand p : getStrands()) {
			b.append(p.getId().toString()).append("/");
		}
		b.append(quoteEnd).append(separator);
		b.append(quoteStart);
		for (Plot p : getPlots()) {
			b.append(p.getId().toString()).append("/");
		}
		b.append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getSummary())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getDescription())).append(quoteStart).append("\n");
		b.append(quoteStart).append(getClean(getNotes())).append(quoteStart).append("\n");
		b.append(quoteStart).append(getClean(getScenario_pitch())).append(quoteStart).append("\n");
		b.append(quoteStart).append(getClean(getScenario_loc())).append(quoteStart).append("\n");
		b.append(quoteStart).append(getClean(getScenario_moment())).append(quoteStart).append("\n");
		b.append(quoteStart).append(getClean(getScenario_start())).append(quoteStart).append("\n");
		b.append(quoteStart).append(getClean(getScenario_end())).append(quoteStart).append("\n");
		return (b.toString());
	}

	@Override
	public String toHtml() {
		return (toCsv("<td>", "</td>", "\n"));
	}

	@Override
	public String toText() {
		return (toCsv("", "", "\t"));
	}

	@Override
	public String toDetail(boolean detailed) {
		//App.trace("Scene.toDetail(detailed=" + (detailed ? "true" : "false") + ")");
		StringBuilder b = new StringBuilder();
		b.append(super.toHtmlDetailHeader(detailed));
		b.append(getInfo(DATA.NUMBER, getClean(getSceneno())));
		b.append(getInfo(DATA.STATUS, Status.getStatusMsg(getStatus())));
		Intensity vi = new Intensity(Intensity.RADIO, getIntensity());
		b.append(getInfo(DATA.INTENSITY, vi.getHtmlValue()));
		SbDuration dur = new SbDuration(getDuration());
		b.append(getInfo(DATA.DURATION, dur.toText(I18N.getMsg("duration.initiales"))));
		if (getChapter() != null) {
			b.append(getInfo(DATA.CHAPTER, getClean(getChapter().getName())));
		}
		if (getStrand() != null) {
			b.append(getInfo(DATA.STRAND, getStrand().getHTMLColor() + " " + getClean(getStrand().getName())));
		}
		if (detailed) {
			b.append(getInfo(DATA.DATE, getClean(getScenets())));
			if (getNarrator() != null) {
				b.append(getInfo(DATA.SCENE_NARRATOR, getClean(getNarrator().id)));
			}
			//status
			b.append(getInfo(DATA.STATUS, getStatus().toString()));
			b.append(getInfo(DATA.INFORMATIVE, getInformative().toString()));
			//duration
			b.append(getInfo(DATA.DURATION, getDurationToText()));
			//intensity
			b.append(getInfo(DATA.INTENSITY, getIntensity().toString()));
			//narrator
			b.append(getInfo(DATA.SCENE_NARRATOR, getClean(getNarrator())));
			if (!getPersons().isEmpty()) {
				b.append(getInfo(DATA.PERSONS, getPersonList()));
			}
			if (!getItems().isEmpty()) {
				b.append(getInfo(DATA.ITEMS, getItemList()));
			}
			if (!getLocations().isEmpty()) {
				b.append(getInfo(DATA.LOCATIONS, getLocationList()));
			}
			if (!getPlots().isEmpty()) {
				b.append(getInfo(DATA.PLOTS, getPlotsList()));
			}
			if (!getStrands().isEmpty()) {
				b.append(getInfo(DATA.STRANDS, getStrandsList()));
			}
			if (getRelativesceneid() > -1L) {
				b.append(getInfo("scene.relative", getClean(getRelativesceneid())));
				SbDuration dx = new SbDuration(getRelativetime());
				b.append(getInfo("scene.relativetime", dx.toText(I18N.getMsg("duration.initiales"))));
			} else {
				b.append(getInfo("scene.relative", ""));
				b.append(getInfo("scene.relativetime", ""));
			}
			//text/summary
			if (getSummary() == null || Html.htmlToText(getSummary()).isEmpty()) {
				b.append(getInfo(DATA.SCENE_SUMMARY, I18N.getMsg("empty")));
			} else {
				String str = " (" + getWords() + " mots)";
				b.append(getInfo(DATA.SCENE_SUMMARY, Html.htmlToText(getText(), 128) + str));
			}
			//xeditor
			if (getOdf() != null && !getOdf().isEmpty()) {
				b.append(getInfo("xeditor.file", getOdf()));
			} else {
				b.append(getInfo("xeditor.file", ""));
			}
			//scenario
			if (getScenario_stage() != 0
					|| !getScenario_pitch().isEmpty()
					|| getScenario_loc() != 0
					|| getScenario_moment() != 0
					|| getScenario_start() != 0
					|| getScenario_end() != 0) {
				b.append(getInfo(DATA.SCENARIO_STAGE, getClean(getScenario_stage())));
				b.append(getInfo(DATA.SCENARIO_PITCH, getClean(getScenario_pitch())));
				b.append(getInfo(DATA.SCENARIO_LOC, getClean(getScenario_loc())));
				b.append(getInfo(DATA.SCENARIO_MOMENT, getClean(getScenario_moment())));
				b.append(getInfo(DATA.SCENARIO_START, getClean(getScenario_start())));
				b.append(getInfo(DATA.SCENARIO_END, getClean(getScenario_end())));
			}
		}
		b.append(super.toHtmlDetailFooter(detailed));
		return (b.toString());
	}

	@Override
	public String toXml() {
		StringBuilder b = new StringBuilder();
		b.append(toXmlTab(1)).append("<scene \n");
		b.append(xmlCommon());
		b.append(xmlAttribute("name", getClean(getTitle())));
		b.append(xmlAttribute("number", getClean(getSceneno())));
		b.append(xmlAttribute("date", getClean(getScenets())));
		b.append(xmlAttribute("status", getClean(getStatus())));
		b.append(xmlAttribute("intensity", getClean(getIntensity())));
		b.append(xmlAttribute("duration", getClean(getDuration())));
		if (getNarrator() != null) {
			b.append(xmlAttribute("narrator", getClean(getNarrator().id)));
		}
		b.append(xmlAttribute("informative", getInformative().toString()));
		if (getChapter() != null) {
			b.append(xmlAttribute("chapter", getClean(getChapter().getId())));
		}
		if (getStrand() != null) {
			b.append(xmlAttribute("pov", getClean(getStrand().getId())));
		}
		b.append(xmlAttribute("stage", getClean(getScenario_stage())));
		b.append(xmlAttribute("pitch", getClean(getScenario_pitch())));
		b.append(xmlAttribute("loc", getClean(getScenario_loc())));
		b.append(xmlAttribute("moment", getClean(getScenario_moment())));
		b.append(xmlAttribute("start", getClean(getScenario_start())));
		b.append(xmlAttribute("end", getClean(getScenario_end())));
		b.append(xmlAttribute("persons", EntityUtil.getListId(persons)));
		b.append(xmlAttribute("items", EntityUtil.getListId(items)));
		b.append(xmlAttribute("locations", EntityUtil.getListId(locations)));
		b.append(xmlAttribute("povs", EntityUtil.getListId(strands)));
		b.append(xmlAttribute("plots", EntityUtil.getListId(plots)));
		b.append(xmlAttribute("relativescene", getClean(getRelativesceneid())));
		//b.append(xmlAttribute("relativedate", getClean(getRelativeDateDifference())));
		b.append(xmlAttribute("relativetime", getClean(getRelativetime())));
		b.append(xmlAttribute("xfile", getOdf()));
		b.append(toXmlTab(2)).append(">\n");
		b.append(toXmlMeta(2, "texte", this.getSummary(), true));
		b.append(toXmlEnd());
		return (b.toString());
	}

	public String toStoryboard() {
		StringBuilder b = new StringBuilder();
		b.append(toXmlTab(1)).append("<shot \n");
		b.append(xmlAttribute("name", String.format("%03d/001", this.getSceneno())));
		b.append(xmlAttribute("seq", this.getSceneno().toString()));
		b.append(xmlAttribute("plan", "1"));
		b.append(xmlAttribute("dur", this.getDuration()));
		/*b.append(xmlAttribute("shot", "0"));*/
		b.append(xmlAttribute("pitch", getClean(getScenario_pitch())));
		b.append(xmlAttribute("loc", getClean(getScenario_loc())));
		b.append(xmlAttribute("moment", getClean(getScenario_moment())));
		b.append(xmlAttribute("start", getClean(getScenario_start())));
		b.append(xmlAttribute("end", getClean(getScenario_end())));
		b.append(toXmlTab(2)).append(">\n");
		b.append(toXmlEnd());
		return (b.toString());
	}

	public static Scene fromXml(Node node) {
		Scene p = new Scene();
		p.setId(getXmlLong(node, "id"));
		p.setName(getXmlText(node, "name"));
		p.setSceneno(getXmlInteger(node, "number"));
		String ts = getXmlString(node, "sceneTs");
		if (!ts.isEmpty()) {
			p.setScenets(Timestamp.valueOf(ts));
		}
		//p.setSbDate(getXmlText(node,"sbDate"));
		p.setStatus(getXmlInteger(node, "status"));
		p.setIntensity(getXmlInteger(node, "intensity"));
		p.setDuration(getXmlString(node, "duration"));
		p.setScenario_pitch(getXmlString(node, "pitch"));
		p.setScenario_loc(getXmlInteger(node, "loc"));
		p.setScenario_moment(getXmlInteger(node, "moment"));
		p.setScenario_start(getXmlInteger(node, "start"));
		p.setScenario_end(getXmlInteger(node, "end"));
		p.setOdf(getXmlText(node, "odf"));
		p.setSummary(getXmlText(node, "summary"));
		p.setDescription(getXmlText(node, "description"));
		p.setNotes(getXmlText(node, "notes"));
		p.setAssistant(getXmlText(node, "assistant"));
		return (p);
	}

	@Override
	@SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
	public boolean equals(Object obj) {
		if (!(obj instanceof Scene)) {
			return false;
		}
		Scene test = (Scene) obj;

		if (id != null ? !id.equals(test.id) : test.id != null) {
			return false;
		}
		boolean ret = equalsStringNullValue(getName(), test.getName());
		ret = ret && equalsIntegerNullValue(sceneno, test.getSceneno());
		ret = ret && equalsIntegerNullValue(status, test.getStatus());
		ret = ret && equalsIntegerNullValue(intensity, test.getIntensity());
		ret = ret && equalsStringNullValue(duration, test.getDuration());
		ret = ret && equalsObjectNullValue(chapter, test.getChapter());
		ret = ret && equalsObjectNullValue(strand, test.getStrand());
		if (scenets != null) {
			ret = ret && equalsTimestampNullValue(scenets, test.getScenets());
		} else if (relativesceneid != null) {
			ret = ret && equalsLongNullValue(relativesceneid, test.getRelativesceneid());
			//ret = ret && equalsIntegerNullValue(relativeDateDifference, test.getRelativeDateDifference());
			ret = ret && equalsStringNullValue(relativetime, test.getRelativetime());
		}
		ret = ret && equalsObjectNullValue(narrator, test.getNarrator());
		ret = ret && equalsStringNullValue(summary, test.getSummary());
		ret = ret && equalsBooleanNullValue(informative, test.getInformative());
		ret = ret && equalsStringNullValue(scenario_pitch, test.getScenario_pitch());
		ret = ret && equalsObjectNullValue(scenario_loc, test.getScenario_loc());
		ret = ret && equalsObjectNullValue(scenario_moment, test.getScenario_moment());
		ret = ret && equalsObjectNullValue(scenario_start, test.getScenario_start());
		ret = ret && equalsObjectNullValue(scenario_end, test.getScenario_end());
		ret = ret && equalsListNullValue(persons, test.getPersons());
		ret = ret && equalsListNullValue(locations, test.getLocations());
		ret = ret && equalsListNullValue(strands, test.getStrands());
		ret = ret && equalsListNullValue(plots, test.getPlots());
		ret = ret && equalsStringNullValue(getDescription(), test.getDescription());
		ret = ret && equalsStringNullValue(getNotes(), test.getNotes());
		return ret;
	}

	@Override
	public int hashCode() {
		return hashPlus(super.hashCode(),
				sceneno,
				status,
				intensity,
				chapter,
				strand,
				strands,
				informative,
				narrator,
				persons,
				items,
				locations,
				plots,
				scenets,
				relativesceneid,
				relativetime,
				scenario_start,
				scenario_end,
				scenario_loc,
				scenario_moment,
				scenario_stage,
				scenario_pitch,
				summary,
				odf);
	}

	@Override
	public int compareTo(AbstractEntity en) {
		return getCCSS().compareTo(((Scene) en).getCCSS());
	}

	public String getOdf() {
		return (odf == null ? "" : odf);
	}

	public void setOdf(String nf) {
		odf = nf;
	}

	public List<Plot> getPlots() {
		return (plots);
	}

	@SuppressWarnings("unchecked")
	public String getPlotsList() {
		return (EntityUtil.getNames((List) plots));
	}

	public void setPlots(List<Plot> plots) {
		this.plots = plots;
	}

	@SuppressWarnings("unchecked")
	public String getItemList() {
		return (EntityUtil.getNames((List) items));
	}

	public static Scene find(List<Scene> list, String str) {
		for (Scene elem : list) {
			if (elem.getName().equals(str)) {
				return (elem);
			}
		}
		return (null);
	}

	public static Scene find(List<Scene> list, Long id) {
		for (Scene elem : list) {
			if (elem.id.equals(id)) {
				return (elem);
			}
		}
		return (null);
	}

	public static Scene findNumber(List scenes, Integer n) {
		for (Object s : scenes) {
			if (((Scene) s).getSceneno().equals(n)) {
				return ((Scene) s);
			}
		}
		return (null);
	}

	public String getScenarioHeader(boolean isScenario) {
		if (!isScenario) {
			return "";
		}
		StringBuilder b = new StringBuilder();
		b.append("<p>");
		if (scenario_loc > 0) {
			b.append(" ");
			b.append(I18N.getMsg(String.format(STD_FMT, "scenario.loc", scenario_loc)));
		} else {
			b.append(" ");
			b.append(I18N.getMsg(String.format(STD_FMT, "scenario.loc", 2)));
		}
		if (scenario_moment > 0) {
			b.append(" ");
			b.append(I18N.getMsg(String.format(STD_FMT, "scenario.moment", scenario_moment)));
		}
		if (locations != null && !locations.isEmpty()) {
			b.append(" - ");
			b.append(locations.get(0).getName().toUpperCase());
		}
		if (scenario_start > 0) {
			b.append(" - ");
			b.append(I18N.getMsg(String.format(STD_FMT, "scenario.in", scenario_start)));
		}
		b.append("</p>");
		return (b.toString());
	}

	public String getScenarioFooter() {
		StringBuilder b = new StringBuilder();
		b.append("<p style=\"text-align:right;\">");
		if (scenario_end > 0) {
			b.append(I18N.getMsg(String.format(STD_FMT, "scenario.out", scenario_end)));
		}
		b.append("</p>");
		return (b.toString());
	}

	public String getScenariopitch() {
		return getScenario_pitch();
	}

	public String getScenariostage() {
		return Codes.getStage(getScenario_stage());
	}

	public String getScenarioloc() {
		return Codes.getLoc(getScenario_loc());
	}

	public String getScenariomoment() {
		return Codes.getMom(getScenario_moment());
	}

	public String getScenariostart() {
		return Codes.findScenarioKey("scenario.in", getScenario_start());
	}

	public String getScenarioend() {
		return Codes.findScenarioKey("scenario.out", getScenario_end());
	}

	public static List<String> getDefColumns() {
		List<String> list = AbstractEntity.getDefColumns(Book.TYPE.SCENE);
		list.add("title, 2048");
		list.add("summary, 32768");
		list.add("scenario_pitch, 1024");
		list.add("odf, 256");
		return (list);
	}

	public static List<String> getTable() {
		List<String> ls = new ArrayList<>();
		String tableName = "scene";
		AbstractEntity.getTable(tableName, ls);
		ls.add(tableName + ",chapter_id,Integer,0");
		ls.add(tableName + ",strand_id,Integer,0");
		ls.add(tableName + ",sceneno,Integer,0");
		ls.add(tableName + ",scene_ts,Time,0");
		ls.add(tableName + ",title,String,2048");
		ls.add(tableName + ",summary,String,32768");
		ls.add(tableName + ",status,Integer,0");
		ls.add(tableName + ",intensity,Integer,0");
		ls.add(tableName + ",duration,String,32");
		ls.add(tableName + ",relative_scene_id,Integer,0");
		ls.add(tableName + ",relative_date_timee,String,255");
		//ls.add(tableName + ",relative_date_difference,Integer,0");
		ls.add(tableName + ",informative,Boolean,0");
		ls.add(tableName + ",scenario_pitch,String,1024");
		ls.add(tableName + ",scenario_stage,Integer,0");
		ls.add(tableName + ",scenario_start,Integer,0");
		ls.add(tableName + ",scenario_end,Integer,0");
		ls.add(tableName + ",scenario_moment,Integer,0");
		ls.add(tableName + ",scenario_loc,Integer,0");
		ls.add(tableName + ",persons,Table.Person,0");
		ls.add(tableName + ",locations,Table.Location,0");
		ls.add(tableName + ",items,Table.Item,0");
		ls.add(tableName + ",strands,Table.Strand,0");
		ls.add(tableName + ",plots,Table.Plot,0");
		ls.add(tableName + ",odf,String,256");
		ls.add(tableName + ",narrator_id,Integer,0");
		return (ls);
	}

	/**
	 * get strands list in HTML format
	 *
	 * @return
	 */
	public String getStrandsHtml() {
		StringBuilder b = new StringBuilder(Html.HTML_B);
		b.append(Html.BODY_B);
		b.append("<table border=\"0\" cellpadding=\"0\" cellspacing=\"1\">");
		for (Strand s : strands) {
			b.append(Html.TR_B);
			b.append(Html.intoTD("█", "color=\"" + ColorUtil.getHTML(s.getJColor()) + "\""));
			b.append(Html.intoTD("&nbsp;"));
			b.append(Html.intoTD(s.getName()));
			b.append(Html.TR_E);
		}
		b.append(Html.TABLE_E);
		b.append(Html.BODY_E);
		b.append(Html.HTML_E);
		return b.toString();
	}

	/**
	 * get the identification like PP.CC.SS where:
	 * <ul>
	 * <li>PP is the Part ID (may be 00 if there is no Part)</li>
	 * <li>CC is the Chapter ID (may be 00 if there is no Chapter)</li>
	 * <li>SS is the Scene ID</li>
	 * </ul>
	 *
	 * @return
	 */
	public String getIdent() {
		if (hasChapter()) {
			return String.format("%s.%02d", chapter.getIdent(), getId());
		}
		return String.format("00.00.%02d", getId());
	}

	/**
	 * get a HTML file name for the scene
	 *
	 * @param scene
	 * @param bookTitle: empty seems it is a unique HTMLfile
	 * @return
	 */
	public static String getFilename(Scene scene, String bookTitle) {
		if (bookTitle.isEmpty()) {
			return "";
		}
		return String.format("%s-%s.html",
				bookTitle,
				scene.getCCSS().replace(".", "_"));
	}

}
