/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.tools.swing;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import javax.swing.JTree;
import javax.swing.tree.TreePath;

/**
 * @author martin
 *
 */
public class TreeUtil {

	private static final String TT = "TreeUtil";

	private TreeUtil() {
		// empty
	}

	public static boolean isDescendant(TreePath path1, TreePath path2) {
		int count1 = path1.getPathCount();
		int count2 = path2.getPathCount();
		if (count1 <= count2) {
			return false;
		}
		while (count1 != count2) {
			path1 = path1.getParentPath();
			count1--;
		}
		return path1.equals(path2);
	}

	public static String expansionStateGet(JTree tree, int row) {
		//App.trace(TT + ".expansionStateGet(tree, row="+row+")");
		TreePath rowPath = tree.getPathForRow(row);
		StringBuilder buf = new StringBuilder();
		int rowCount = tree.getRowCount();
		for (int i = row; i < rowCount; i++) {
			TreePath path = tree.getPathForRow(i);
			if (i == row || isDescendant(path, rowPath)) {
				if (tree.isExpanded(path)) {
					buf.append(",").append(String.valueOf(i - row));
				}
			} else {
				break;
			}
		}
		return buf.toString();
	}

	public static void expanstionStateSet(JTree tree, int row, String state) {
		//App.trace(TT + ".expansionStateSet(tree, row="+row+", state="+state+")");
		StringTokenizer stok = new StringTokenizer(state, ",");
		while (stok.hasMoreTokens()) {
			int token = row + Integer.parseInt(stok.nextToken());
			tree.expandRow(token);
		}
	}

	public static List<String> expansionSave(JTree tree) {
		//App.trace(TT + ".expansionSave(tree)");
		List<String> list = new ArrayList<>();
		int rowCount = tree.getRowCount();
		for (int i = 0; i < rowCount; i++) {
			TreePath path = tree.getPathForRow(i);
			if (tree.isExpanded(path)) {
				list.add(path.toString());
				//App.trace("save: " + path.toString());
			}
		}
		return list;
	}

	public static void expansionRestore(JTree tree, List<String> list) {
		//App.trace(TT + ".expansionRestore(tree)");
		int rowCount = tree.getRowCount();
		for (String s : list) {
			for (int i = 0; i < rowCount; i++) {
				TreePath path = tree.getPathForRow(i);
				if (s.equals(path.toString())) {
					tree.expandRow(i);
					rowCount = tree.getRowCount();
				}
			}
		}
	}

}
