/*
 * Copyright (C) 2018 FaVdB
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.tools.xml;

import java.io.IOException;
import java.io.StringReader;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.SbDate;
import storybook.tools.LOG;

/**
 *
 * @author FaVdB
 */
public class XmlUtil {

	public static Element createChild(Document doc, Node parent, String tag, String... attribs) {
		/*App.trace("XmlUtil" + ".createChild("
				+ "parent=" + parent.getNodeName()
				+ ", tag=" + tag
				+ ", attribs nb="
				+ (attribs == null ? "0" : attribs.length)
				+ ")");*/
		Element child = doc.createElement(tag);
		if (attribs != null && attribs.length > 0) {
			for (String a : attribs) {
				if (a.contains("=")) {
					String v[] = a.split("=");
					child.setAttribute(v[0], v[1].replace("\"", ""));
				}
			}
		}
		parent.appendChild(child);
		return child;
	}

	private XmlUtil() {
		// empty
	}

	public static Element getRootNode(String str) {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document doc = builder.parse(new InputSource(new StringReader(str)));
			Element rootNode = doc.getDocumentElement();
			return (rootNode);
		} catch (ParserConfigurationException | SAXException | IOException ex) {
			LOG.err("TempUtil restore failed", ex);
			return (null);
		}
	}

	public static String NodeToString(Node node) {
		StringBuilder b = new StringBuilder();
		getXMLString(node, true, b, true);
		return b.toString();
	}

	public static void getXMLString(Node node, boolean withoutNamespaces, StringBuilder buff, boolean endTag) {
		buff.append("<").append(node.getNodeName());
		if (node.hasAttributes()) {
			buff.append(" ");
			NamedNodeMap attr = node.getAttributes();
			for (int i = 0; i < attr.getLength(); i++) {
				Node attrItem = attr.item(i);
				String name = attrItem.getNodeName();
				String value = attrItem.getNodeValue();
				buff.append(name).append("=\"").append(value).append("\"");

				if (i < attr.getLength() - 1) {
					buff.append(" ");
				}
			}
		}
		if (node.hasChildNodes()) {
			buff.append(">");
			NodeList children = node.getChildNodes();
			int childrenCount = children.getLength();
			if (childrenCount == 1) {
				Node item = children.item(0);
				int itemType = item.getNodeType();
				if (itemType == Node.TEXT_NODE) {
					if (item.getNodeValue() == null) {
						buff.append("/>");
					} else {
						buff.append(item.getNodeValue());
						buff.append("</")
								.append(node.getNodeName())
								.append(">");
					}
					endTag = false;
				}
			}
			for (int i = 0; i < childrenCount; i++) {
				Node item = children.item(i);
				int itemType = item.getNodeType();
				if (itemType == Node.DOCUMENT_NODE || itemType == Node.ELEMENT_NODE) {
					getXMLString(item, withoutNamespaces, buff, endTag);
				}
			}
		} else {
			if (node.getNodeValue() == null) {
				buff.append("/>");
			} else {
				buff.append(node.getNodeValue());
				buff.append("</").append(node.getNodeName()).append(">");
			}
			endTag = false;
		}
		if (endTag) {
			buff.append("</").append(node.getNodeName()).append(">");
		}
	}

	/**
	 * get attribute value of a Node
	 *
	 * @param node: the Node
	 * @param key: the attribute key name
	 * @return
	 */
	public static String getAttribute(Node node, String key) {
		Element e = (Element) node;
		return (e.getAttribute(key).trim());
	}

	/**
	 * get attribute value of a String Node
	 *
	 * @param node: the Node String
	 * @param key: the attribute key name
	 * @return
	 */
	public static String getAttribute(String node, String key) {
		int hstart = node.indexOf(key) + key.length();
		String sx = node.substring(hstart);
		sx = sx.substring(0, sx.indexOf("\""));
		return sx;
	}

	public static void attributeSet(Node node, String key, Object value) {
		if (node == null || key == null || key.isEmpty()) {
			return;
		}
		String val;
		if (value instanceof String
				|| value instanceof Long
				|| value instanceof Integer
				|| value instanceof Boolean) {
			val = value.toString();
		} else if (value instanceof AbstractEntity) {
			val = ((AbstractEntity) value).getName();
		} else if (value instanceof SbDate) {
			val = ((SbDate) value).getDateTimeToString();
		} else {
			val = null;
		}
		Element e = (Element) node;
		if (e.getAttribute(key) != null) {
			if (val != null && !val.isEmpty()) {
				e.setAttribute(key, val);
			} else {
				e.removeAttribute(key);
			}
			return;
		}
		if (val == null || val.isEmpty()) {
			return;
		}
		Node nv = node.getAttributes().getNamedItem(key);
		if (nv != null) {
			nv.setNodeValue(val);
		}
	}

	public static void attributeRemove(Node node, String key) {
		Element e = (Element) node;
		if (e == null) {
			return;
		}
		if (e.getAttribute(key) != null) {
			e.removeAttribute(key);
		}
	}

}
