/*
 * SbApp: Open Source software for novelists and authors.
 * Original idea 2008 - 2012 Martin Mustun
 * Copyrigth (C) Favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
package storybook.ui.combobox;

import assistant.Assistant;
import i18n.I18N;
import java.util.Collections;
import java.util.List;
import javax.swing.DefaultListModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import org.hibernate.Session;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.AttributeDAO;
import storybook.model.hbn.dao.CategoryDAO;
import storybook.model.hbn.dao.ChapterDAO;
import storybook.model.hbn.dao.GenderDAO;
import storybook.model.hbn.dao.IdeaDAO;
import storybook.model.hbn.dao.ItemDAO;
import storybook.model.hbn.dao.LocationDAO;
import storybook.model.hbn.dao.PartDAO;
import storybook.model.hbn.dao.PersonDAO;
import storybook.model.hbn.dao.SceneDAO;
import storybook.model.hbn.dao.StrandDAO;
import storybook.model.hbn.dao.TagDAO;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Attribute;
import storybook.model.hbn.entity.Category;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.Gender;
import storybook.model.hbn.entity.Idea;
import storybook.model.hbn.entity.Item;
import storybook.model.hbn.entity.Location;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Status;
import storybook.model.hbn.entity.Strand;
import storybook.model.hbn.entity.Tag;
import storybook.ui.MainFrame;
import storybook.ui.renderer.lcr.LCREntity;

/**
 *
 * @author favdb
 */
public class ComboUtil {

	private ComboUtil() {
		// empty
	}

	public static boolean NEW = true, EMPTY = true, ALL = true;

	@SuppressWarnings("unchecked")
	public static void loadCbChapters(MainFrame mainFrame, JComboBox cb, Scene scene) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ChapterDAO dao = new ChapterDAO(session);
		List<Chapter> chapters = dao.findAll();
		cb.removeAllItems();
		int ix = -1, i = 0;
		for (Chapter chapter : chapters) {
			cb.addItem(chapter.getPart() + "." + chapter.getChapternoStr() + ". " + chapter.getName());
			if ((scene.hasChapter()) && (scene.getChapter().equals(chapter))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbStage(MainFrame mainFrame, JComboBox cb, String toSel) {
		cb.removeAllItems();
		cb.addItem("");//empty item
		List<String> list = Assistant.getListOf("stage");
		if (list == null || list.isEmpty()) {
			return;
		}
		for (String elem : list) {
			cb.addItem(elem);
		}
		if (toSel != null && !toSel.isEmpty()) {
			cb.setSelectedItem(toSel);
		}
	}

	@SuppressWarnings("unchecked")
	public static void loadCbStatus(JComboBox cb, int sel) {
		cb.removeAllItems();
		if (sel == -1) {
			cb.addItem(I18N.getMsg("change.no"));
		}
		for (int i = 0; i < Status.STATUS.values().length; i++) {
			cb.addItem(i);
		}
		if (sel != -1 && sel < cb.getItemCount()) {
			cb.setSelectedIndex(sel);
		}
	}

	@SuppressWarnings("unchecked")
	public static void loadLbStrands(MainFrame mainFrame, JList lb, Scene scene) {
		DefaultListModel listModel = new DefaultListModel();
		if (!"".equals(scene.getTitle())) {
			Model model = mainFrame.getBookModel();
			Session session = model.beginTransaction();
			StrandDAO dao = new StrandDAO(session);
			List<Strand> strands = dao.findAll();
			int ix = -1, i = 0;
			for (Strand strand : strands) {
				listModel.addElement(strand.getName());
				if ((scene.getStrand() != null) && (scene.getStrand().equals(strand))) {
					ix = i;
				}
				i++;
			}
			lb.setModel(listModel);
			lb.setSelectedIndex(ix);
			model.commit();
		} else {
			lb.setModel(listModel);
		}
	}

	@SuppressWarnings("unchecked")
	public static void fillCbGenders(MainFrame mainFrame, JComboBox cb) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		GenderDAO dao = new GenderDAO(session);
		List<Gender> genders = dao.findAll();
		cb.removeAllItems();
		for (Gender gender : genders) {
			cb.addItem(gender);
		}
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbGenders(MainFrame mainFrame, JComboBox cb, Person person) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		GenderDAO dao = new GenderDAO(session);
		List<Gender> genders = dao.findAll();
		cb.removeAllItems();
		int ix = -1, i = 0;
		for (Gender gender : genders) {
			cb.addItem(gender.getName());
			if ((person != null) && ((!person.getFullName().equals(" ")) && (person.getGender().equals(gender)))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	public static Gender findGender(MainFrame mainFrame, String str) {
		Gender rgender = null;
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		GenderDAO dao = new GenderDAO(session);
		List<Gender> genders = dao.findAll();
		for (Gender gender : genders) {
			if (str.equals(gender.getName())) {
				rgender = gender;
				break;
			}
		}
		model.commit();
		return (rgender);
	}

	@SuppressWarnings("unchecked")
	public static void loadLbPersons(MainFrame mainFrame, JList lb, Scene scene) {
		DefaultListModel listModel = new DefaultListModel();
		if (!"".equals(scene.getTitle())) {
			Model model = mainFrame.getBookModel();
			Session session = model.beginTransaction();
			PersonDAO dao = new PersonDAO(session);
			List<Person> persons = dao.findAll();
			int i = 0;
			int[] indices = {};
			for (Person person : persons) {
				listModel.addElement(person.getFullName());
				if ((scene.getPersons() != null) && (scene.getPersons().contains(person))) {
					indices[indices.length] = i;
				}
				i++;
			}
			lb.setModel(listModel);
			lb.setSelectedIndices(indices);
			model.commit();
		} else {
			lb.setModel(listModel);
		}
	}

	public static boolean findPersonAbbreviation(MainFrame mainFrame, String str) {
		if (!"".equals(str)) {
			return (false);
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PersonDAO dao = new PersonDAO(session);
		List<Person> persons = dao.findAll();
		boolean r = false;
		for (Person person : persons) {
			if (person.getAbbreviation().equals(str)) {
				r = true;
				break;
			}
		}
		model.commit();
		return (r);
	}

	@SuppressWarnings("unchecked")
	public static void loadCbLocations(MainFrame mainFrame, JComboBox cb, Scene scene) {
		if (!"".equals(scene.getTitle())) {
			Model model = mainFrame.getBookModel();
			Session session = model.beginTransaction();
			LocationDAO dao = new LocationDAO(session);
			List<Location> locations = dao.findAll();
			int ix = -1;
			int i = 0;
			cb.removeAllItems();
			List<Location> sceneloc = scene.getLocations();
			for (Location location : locations) {
				cb.addItem(location.getFullName());
				if ((sceneloc.contains(location))) {
					ix = i;
				}
				i++;
			}
			cb.setSelectedIndex(ix);
			model.commit();
		}
	}

	@SuppressWarnings("unchecked")
	public static void loadCbParts(MainFrame mainFrame, JComboBox cb, Chapter chapter) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PartDAO dao = new PartDAO(session);
		List<Part> parts = dao.findAll();
		cb.removeAllItems();
		int ix = -1;
		int i = 0;
		for (Part part : parts) {
			cb.addItem(part.getNumberName());
			if ((chapter.hasPart()) && (chapter.getPart().equals(part))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	public static Part findPart(MainFrame mainFrame, String str) {
		if ("".equals(str)) {
			return (null);
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PartDAO dao = new PartDAO(session);
		List<Part> parts = dao.findAll();
		Part rpart = null;
		for (Part part : parts) {
			if (part.getNumberName().equals(str)) {
				rpart = part;
				break;
			}
		}
		model.commit();
		return (rpart);
	}

	@SuppressWarnings("unchecked")
	public static void loadCbCategories(MainFrame mainFrame, JComboBox cb) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		CategoryDAO dao = new CategoryDAO(session);
		List<Category> categories = dao.findAll();
		cb.removeAllItems();
		for (Category category : categories) {
			cb.addItem(category.getName());
		}
		cb.setSelectedIndex(-1);
		model.commit();
	}

	public static Category getCbCategory(MainFrame mainFrame, JComboBox cb) {
		Category rcategory = null;
		if (cb.getSelectedIndex() == -1) {
			return (rcategory);
		}
		String str = cb.getSelectedItem().toString();
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		CategoryDAO dao = new CategoryDAO(session);
		List<Category> categories = dao.findAll();
		for (Category category : categories) {
			if (category.getName().equals(str)) {
				rcategory = category;
				break;
			}
		}
		model.commit();
		return (rcategory);
	}

	@SuppressWarnings("unchecked")
	public static void loadCbCategories(MainFrame mainFrame, JComboBox cb, Idea idea) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		IdeaDAO dao = new IdeaDAO(session);
		List<String> ideas = dao.findCategories();
		cb.removeAllItems();
		int ix = -1;
		int i = 0;
		String strIdea = (idea != null ? idea.getCategory() : "");
		for (String str : ideas) {
			cb.addItem(str);
			if ((idea != null) && (!strIdea.equals("")) && (strIdea.equals(str))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadLbAttributes(MainFrame mainFrame, JList lb, Person person) {
		DefaultListModel listModel = new DefaultListModel();
		if (person.getAttributes() != null) {
			Model model = mainFrame.getBookModel();
			Session session = model.beginTransaction();
			AttributeDAO dao = new AttributeDAO(session);
			List<Attribute> attributes = dao.findAll();
			int ix = -1;
			int i = 0;
			for (Attribute attribute : attributes) {
				listModel.addElement(attribute.getKey() + ":" + attribute.getValue());
				if ((!person.getFullName().equals("")) && (attribute.equals(attribute))) {
					ix = i;
				}
				i++;
			}
			lb.setModel(listModel);
			lb.setSelectedIndex(ix);
			model.commit();
		} else {
			lb.setModel(listModel);
		}
	}

	@SuppressWarnings("unchecked")
	public static void loadCbCities(MainFrame mainFrame, JComboBox cb, Location location) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		LocationDAO dao = new LocationDAO(session);
		List<String> cities = dao.findCities();
		cb.removeAllItems();
		int ix = -1;
		int i = 0;
		for (String city : cities) {
			cb.addItem(city);
			if ((location != null) && (!location.getId().equals(new Long(-1))) && (location.getCity().equals(city))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	public static void loadCbSites(MainFrame mainFrame, JComboBox<Location> cb, Location location) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		LocationDAO dao = new LocationDAO(session);
		List<Location> sites = dao.findAll();
		cb.removeAllItems();
		for (Location site : sites) {
			cb.addItem(site);
		}
		if (location != null && location.getSite() != null) {
			cb.setSelectedItem(location.getSite());
		}
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbCountries(MainFrame mainFrame, JComboBox cb, Location location) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		LocationDAO dao = new LocationDAO(session);
		List<String> countries = dao.findCountries();
		cb.removeAllItems();
		for (String country : countries) {
			cb.addItem(country);
		}
		if (location != null && !location.getCountry().isEmpty()) {
			cb.setSelectedItem(location.getCountry());
		}
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbCategories(MainFrame mainFrame, JComboBox cb, Item item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemDAO dao = new ItemDAO(session);
		List<String> categories = dao.findCategories();
		cb.removeAllItems();
		for (String category : categories) {
			cb.addItem(category);
		}
		if (item != null && !item.getCategory().isEmpty()) {
			cb.setSelectedItem(item.getCategory());
		}
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbCategories(MainFrame mainFrame, JComboBox cb, Tag tag) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TagDAO dao = new TagDAO(session);
		List<String> categories = dao.findCategories();
		cb.removeAllItems();
		for (String category : categories) {
			cb.addItem(category);
		}
		if (tag != null && !tag.getCategory().isEmpty()) {
			cb.setSelectedItem(tag.getCategory());
		}
		model.commit();
	}

	public static boolean isMultiLbContains(JList lb, List ls) {
		List lx = lb.getSelectedValuesList();
		if (lx.equals(ls)) {
			return (true);
		}
		return (false);
	}

	public static boolean isCbEquals(JComboBox cb, String str) {
		if (cb.getSelectedIndex() == -1) {
			if (str.equals("")) {
				return (true);
			}
		} else {
			if (str.equals("")) {
				return (false);
			}
			if (cb.getSelectedItem().equals(str)) {
				return (true);
			}
		}
		return (false);
	}

	@SuppressWarnings("unchecked")
	public static void loadCbTags(MainFrame mainFrame, JComboBox cb, Tag tag) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TagDAO dao = new TagDAO(session);
		List<Tag> tags = dao.findAll();
		cb.removeAllItems();
		int ix = -1;
		int i = 0;
		for (Tag u : tags) {
			cb.addItem((String) u.getName());
			if ((tag != null) && (!tag.getName().isEmpty()) && (tag.getName().equals(u.getName()))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	public static Tag findTag(MainFrame mainFrame, String str) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		TagDAO dao = new TagDAO(session);
		List<Tag> tags = dao.findAll();
		Tag r = null;
		for (Tag u : tags) {
			if (u.getName().equals(str)) {
				r = u;
				break;
			}
		}
		model.commit();
		return (r);
	}

	@SuppressWarnings("unchecked")
	public static void loadCbItems(MainFrame mainFrame, JComboBox cb, Item item) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		ItemDAO dao = new ItemDAO(session);
		List<Item> items = dao.findAll();
		cb.removeAllItems();
		int ix = -1;
		int i = 0;
		for (Item u : items) {
			cb.addItem((String) u.getName());
			if ((item != null) && (!item.getName().isEmpty()) && (item.getName().equals(u.getName()))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbPersons(MainFrame mainFrame, JComboBox cb, Person person) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		PersonDAO dao = new PersonDAO(session);
		List<Person> persons = dao.findAll();
		cb.removeAllItems();
		int ix = -1;
		int i = 0;
		for (Person u : persons) {
			cb.addItem(u.getFullName());
			if ((person != null) && (!person.toString().isEmpty()) && (person.toString().equals(u.toString()))) {
				ix = i;
			}
			i++;
		}
		cb.setSelectedIndex(ix);
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbLocations(MainFrame mainFrame, JComboBox cb, Location location) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		LocationDAO dao = new LocationDAO(session);
		List<Location> locations = dao.findAll();
		cb.removeAllItems();
		for (Location u : locations) {
			cb.addItem(u.getFullName());
		}
		if ((location != null) && (!location.getFullName().isEmpty())) {
			cb.setSelectedItem(location.getFullName());
		}
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void loadCbScenes(MainFrame mainFrame, JComboBox cb, Scene scene) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAO dao = new SceneDAO(session);
		List<Scene> scenes = dao.findAll();
		cb.removeAllItems();
		for (Scene u : scenes) {
			cb.addItem(u.getFullTitle());
		}
		if (scene != null && !scene.getFullTitle().isEmpty()) {
			cb.setSelectedItem(scene.getFullTitle());
		}
		model.commit();
	}

	@SuppressWarnings("unchecked")
	public static void fillEntity(MainFrame mainFrame, JComboBox cb, Book.TYPE type,
			AbstractEntity toSel, AbstractEntity toHide, String options) {
		List<AbstractEntity> entities = EntityUtil.findEntities(mainFrame, type);
		if (type == Book.TYPE.CATEGORY) {
			Collections.sort(entities, (AbstractEntity e1, AbstractEntity e2)
					-> ((Category) e1).getSort().compareTo(((Category) e2).getSort()));
		} else {
			Collections.sort(entities, (AbstractEntity e1, AbstractEntity e2)
					-> e1.getFullName().compareTo(e2.getFullName()));
		}
		fillEntity(cb, entities, toSel, toHide, options);
	}

	public static void fillEntity(MainFrame mainFrame, JComboBox cb, Book.TYPE type,
			Long toSel, AbstractEntity toHide, String options) {
		@SuppressWarnings("unchecked")
		List<AbstractEntity> entities = EntityUtil.findEntities(mainFrame, type);
		if (type == Book.TYPE.CATEGORY) {
			Collections.sort(entities, (AbstractEntity e1, AbstractEntity e2)
					-> ((Category) e1).getSort().compareTo(((Category) e2).getSort()));
		} else {
			Collections.sort(entities, (AbstractEntity e1, AbstractEntity e2)
					-> e1.getName().compareTo(e2.getName()));
		}
		fillEntity(cb, entities, toSel, toHide, options);
		//cb.revalidate();
	}

	@SuppressWarnings("unchecked")
	public static void fillEntity(JComboBox cb,
			List<AbstractEntity> entities, AbstractEntity toSel, AbstractEntity toHide, String options) {
		cb.removeAllItems();
		cb.setRenderer(new LCREntity());
		if (isAddEmpty(options)) {
			cb.addItem("");
		}
		if (isAddAll(options)) {
			cb.addItem(I18N.getMsg("all"));
		}
		entities.forEach((entity) -> {
			if (toHide == null || !entity.equals(toHide)) {
				cb.addItem(entity);
			}
		});
		if (toSel != null) {
			cb.setSelectedItem(toSel);
		}
		cb.revalidate();
	}

	@SuppressWarnings("unchecked")
	public static void fillEntity(JComboBox cb,
			List<AbstractEntity> entities, Long toSel, AbstractEntity toHide, String options) {
		cb.removeAllItems();
		cb.setRenderer(new LCREntity());
		if (isAddEmpty(options)) {
			cb.addItem("");
		}
		if (isAddAll(options)) {
			cb.addItem(I18N.getMsg("all"));
		}
		int s = -1, i = 0;
		for (AbstractEntity entity : entities) {
			if (toHide == null || !entity.equals(toHide)) {
				cb.addItem(entity);
				if (entity.getId().equals(toSel)) {
					s = i;
				}
				i++;
			}
		}
		if (s != -1) {
			cb.setSelectedItem(s);
		}
		cb.revalidate();
	}

	private static boolean isAddNew(String x) {
		return x.charAt(0) == '1';
	}

	private static boolean isAddEmpty(String x) {
		return x.charAt(1) == '1';
	}

	private static boolean isAddAll(String x) {
		return x.charAt(2) == '1';
	}

}
