/*
 * Copyright (C) 2015 favdb
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package storybook.ui.panel.memo;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.util.List;
import javax.swing.DefaultComboBoxModel;
import javax.swing.DefaultListModel;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import net.infonode.docking.View;
import org.hibernate.Session;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import shef.ShefEditor;
import storybook.App;
import storybook.Pref;
import storybook.action.DeleteEntityAction;
import storybook.ctrl.ActKey;
import storybook.ctrl.Ctrl;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.MemoDAO;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Memo;
import storybook.tools.net.Net;
import storybook.tools.swing.SwingUtil;
import storybook.tools.swing.js.JSToolBar;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.Ui;
import storybook.ui.panel.AbstractPanel;

/**
 *
 * @author favdb
 */
public class MemosPanel extends AbstractPanel
		implements ActionListener,
		ListSelectionListener, HyperlinkListener,
		MouseListener, ItemListener {

	private static final String TT = "MemosPanel";
	private boolean saveDisposition;
	private ShefEditor shef;
	private JComboBox memoCombo = null;//liste deroulante des memos
	private JList memoList = null;//liste des memos
	private JButton btNew;// bouton nouveau
	private JButton btDelete;// bouton supprimer
	private JButton btEdit;// bouton modifier
	//private JTextPane infoPanel;// le panneau d'information
	private Long currentMemoId;// le memo actuellement affiché
	private boolean dispoCombo = false;
	private JScrollPane listScroller;
	private JCheckBox ckDisposition;

	private enum ACT {
		BT_NEW, BT_DELETE, BT_EDIT, CK_DISPOSITION, CB_MEMO, LS_MEMO, NONE;

		public String n() {
			return name().toLowerCase();
		}
	}

	private ACT getACT(String s) {
		for (ACT a : ACT.values()) {
			if (a.n().equals(s)) {
				return a;
			}
		}
		return ACT.NONE;
	}

	public MemosPanel(MainFrame mainFrame) {
		super(mainFrame);
	}

	@Override
	public void init() {
		//LOG.trace(TT + ".init()");
		this.withPart = false;
		dispoCombo = App.preferences.getBoolean(Pref.KEY.MEMO_LAYOUT_DIRECTION);
		currentMemoId = null;
		this.resetListener();
	}

	@Override
	public void initUi() {
		//LOG.trace(TT + ".initUi()");
		setLayout(new MigLayout(MIG.get(MIG.WRAP, MIG.FILL, MIG.HIDEMODE3), "[20%][80%]", "[][grow][]"));
		add(initToolbar(), MIG.get(MIG.SPAN, MIG.WRAP, MIG.GROWX));
		ckDisposition = Ui.initCheckBox(ACT.CK_DISPOSITION.n(), "memo.layoutdirection.top", dispoCombo, this);
		toolbar.add(ckDisposition, MIG.RIGHT);
		memoCombo = new JComboBox();
		memoCombo.setName(ACT.CB_MEMO.n());
		memoCombo.setMinimumSize(new Dimension(128, memoCombo.getPreferredSize().height));
		toolbar.add(memoCombo);
		memoList = new JList<>();
		memoList.setName(ACT.LS_MEMO.n());
		refreshListMemo();
		memoList.addMouseListener(this);
		if (memoCombo.getItemCount() > 0) {
			memoList.setSelectedIndex(0);
		}
		listScroller = new JScrollPane(memoList);
		SwingUtil.setMaxPreferredSize(listScroller);
		add(listScroller);
		//memo as a Shef
		shef = new ShefEditor("reduced, disallow", "");
		add(shef, MIG.get(MIG.SPAN, MIG.GROW));

		JSToolBar footing = new JSToolBar(true);
		btNew = Ui.initButton(ACT.BT_NEW.n(), "", ICONS.K.NEW, "new", this);
		footing.add(btNew);
		btEdit = Ui.initButton(ACT.BT_EDIT.n(), "", ICONS.K.EDIT, "edit", this);
		btEdit.setEnabled(false);
		footing.add(btEdit);
		btDelete = Ui.initButton(ACT.BT_DELETE.n(), "", ICONS.K.DELETE, "delete", this);
		btDelete.setEnabled(false);
		footing.add(btDelete);
		add(footing, MIG.get(MIG.GROWX, MIG.SPAN));
		setListener();
		setDisposition();
		refreshMemo();
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		//LOG.trace(TT + ".modelPropertyChange(evt=" + evt.toString() + ")");
		Object newValue = evt.getNewValue();
		String propName = evt.getPropertyName();
		switch (Ctrl.getPROPS(propName)) {
			case REFRESH: {
				View newView = (View) newValue;
				View view = (View) getParent().getParent();
				if (view == newView) {
					refresh();
				}
				return;
			}
			case MEMO_SHOW: {
				if (newValue instanceof AbstractEntity) {
					currentMemoId = ((Memo) newValue).getId();
					Memo memo = getMemo(currentMemoId);
					if (memo.isTransient()) {
						return;
					}
					Model model = mainFrame.getBookModel();
					Session session = model.beginTransaction();
					session.refresh(currentMemoId);
					model.commit();
					setMemo(currentMemoId);
					return;
				}
				break;
			}
			case MEMO_LAYOUTDIRECTION: {
				int idx = (memoList != null ? memoList.getSelectedIndex() : memoCombo.getSelectedIndex());
				refresh();
				if (!dispoCombo) {
					memoList.setSelectedIndex(idx);
				} else {
					memoCombo.setSelectedIndex(idx);
				}
				return;
			}
			case SHOWINFO:
				if (newValue instanceof Memo) {
					setMemo(((Memo) newValue).getId());
				}
				dispoCombo = saveDisposition;
				return;
			default:
				break;
		}
		if (currentMemoId != null && newValue instanceof AbstractEntity) {
			AbstractEntity updatedEntity = (AbstractEntity) newValue;
			if (updatedEntity.getId().equals(currentMemoId)) {
				refreshMemo();
			}
		} else {
			ActKey act = new ActKey(evt);
			if (Book.getTYPE(act.type) == Book.TYPE.MEMO && !Ctrl.PROPS.INIT.check(act.getCmd())) {
				if (newValue != null) {
					currentMemoId = ((Memo) newValue).getId();
				} else {
					currentMemoId = -1L;
				}
				refresh();
			}
		}
	}

	@Override
	public void hyperlinkUpdate(HyperlinkEvent evt) {
		if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			Net.openUrl(evt);
		}
	}

	@Override
	public void actionPerformed(ActionEvent evt) {
		//LOG.trace(TT + ".actionPerformed(evt=" + evt.toString() + ")");
		if (evt.getSource() instanceof JComboBox) {
			if (memoCombo.getSelectedIndex() != -1) {
				currentMemoId = ((Memo) memoCombo.getSelectedItem()).getId();
			} else {
				currentMemoId = -1L;
			}
			refreshMemo();
			return;
		}
		if (evt.getSource() instanceof JButton) {
			saveDisposition = dispoCombo;
			Memo memo = getMemo(currentMemoId);
			switch (getACT(((JButton) evt.getSource()).getName())) {
				case BT_EDIT:
					resetListener();
					mainFrame.showEditorAsDialog(memo);
					//refreshListMemo();
					return;
				case BT_DELETE:
					resetListener();
					DeleteEntityAction act = new DeleteEntityAction(mainFrame, memo);
					act.actionPerformed(null);
					/*if (act.isConfirm()) {
						if (!dispoCombo) {
							memoList.setSelectedIndex(-1);
						} else {
							memoCombo.setSelectedIndex(-1);
						}
						refreshListMemo();
					}*/
					return;
				case BT_NEW:
					resetListener();
					mainFrame.showEditorAsDialog(new Memo());
					return;
				default:
					break;
			}
		}
		if (evt.getSource() instanceof JCheckBox) {
			dispoCombo = ckDisposition.isSelected();
			App.preferences.setBoolean(Pref.KEY.MEMO_LAYOUT_DIRECTION, dispoCombo);
			setDisposition();
		}
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {
		//LOG.trace(TT + ".valueChanged(e=" + e.toString() + ")");
		checkTextChanged();
		if (!dispoCombo) {
			if (memoList.getSelectedValue() instanceof Memo) {
				currentMemoId = ((Memo) memoList.getSelectedValue()).getId();
			}
		} else {
			if (memoCombo.getSelectedItem() instanceof Memo) {
				currentMemoId = ((Memo) memoCombo.getSelectedItem()).getId();
			}
		}
		if (currentMemoId != null) {
			Memo memo = getMemo(currentMemoId);
			memoCombo.setSelectedItem(memo);
			mainFrame.getBookController().showInfo(memo);
		}
		refreshMemo();
	}

	private void setListener() {
		memoList.addListSelectionListener(this);
		memoCombo.addActionListener(this);
		ckDisposition.addActionListener(this);
	}

	private void resetListener() {
		if (memoList != null) {
			memoList.removeListSelectionListener(this);
		}
		if (memoCombo != null) {
			memoCombo.removeActionListener(this);
		}
		if (ckDisposition != null) {
			ckDisposition.removeActionListener(this);
		}
	}

	private void refreshMemo() {
		//LOG.trace(TT + ".refreshMemo() currentMemo=" + LOG.traceEntity(currentMemo) + ")");
		if (shef == null) {
			return;
		}
		resetListener();
		shef.setCaretPosition(0);
		if (currentMemoId == null || currentMemoId == -1L) {
			shef.setText("");
			shef.setEditable(false);
			btEdit.setEnabled(false);
			btDelete.setEnabled(false);
		} else {
			Memo memo = getMemo(currentMemoId);
			shef.setText(memo.getNotes());
			shef.setEditable(true);
			btEdit.setEnabled(true);
			btDelete.setEnabled(true);
		}
		setListener();
	}

	@SuppressWarnings({"unchecked", "unchecked", "unchecked", "unchecked"})
	private void refreshListMemo() {
		//LOG.trace(TT + ".refreshListMemo()");
		//remove listeners
		resetListener();
		//memorize selected Memo
		Memo entitySelected = null;
		if (memoCombo.getItemCount() > 1) {
			if (!dispoCombo) {
				entitySelected = (Memo) memoList.getSelectedValue();
			} else {
				entitySelected = (Memo) memoCombo.getSelectedItem();
			}
		}
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		MemoDAO dao = new MemoDAO(session);
		List<Memo> memos = dao.findAll();
		model.commit();
		//reload list
		DefaultListModel list = new DefaultListModel();
		list.removeAllElements();
		for (Memo memo : memos) {
			list.addElement(memo);
		}
		memoList.setModel(list);
		memoList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//reload combo
		DefaultComboBoxModel combo = (DefaultComboBoxModel) memoCombo.getModel();
		combo.removeAllElements();
		//combo.addElement("");
		for (Memo memo : memos) {
			combo.addElement(memo);
		}
		memoCombo.setModel(combo);
		//select prior selected Memo
		if (entitySelected != null) {
			setMemo(entitySelected.getId());
		} else {
			if (currentMemoId != null) {
				setMemo(currentMemoId);
			}
		}
		//reactivate listener
		setListener();
	}

	private void setMemo(Long memoId) {
		if (memoId != null && memoId != -1) {
			Memo memo = getMemo(memoId);
			memoList.setSelectedValue(memo, true);
			memoCombo.setSelectedItem(memo);
			currentMemoId = memoId;
		} else {
			memoCombo.setSelectedIndex(0);
		}
	}

	public JList getMemoList() {
		return memoList;
	}

	private void setDisposition() {
		memoCombo.setVisible(dispoCombo);
		listScroller.setVisible(!dispoCombo);
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		DefaultListSelectionModel selectionModel = (DefaultListSelectionModel) memoList.getSelectionModel();
		int row = selectionModel.getMinSelectionIndex();
		int count = selectionModel.getMaxSelectionIndex() - selectionModel.getMinSelectionIndex() + 1;
		if (count > 1) {
			return;
		}
		if (e.getClickCount() == 2) {
			Memo memo = (Memo) memoList.getModel().getElementAt(row);
			mainFrame.showEditorAsDialog(memo);
			refreshListMemo();
		}
	}

	@Override
	public void mousePressed(MouseEvent e) {
		//empty
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		//empty
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		//empty
	}

	@Override
	public void mouseExited(MouseEvent e) {
		//empty
	}

	@Override
	public void itemStateChanged(ItemEvent e) {
		//LOG.trace(TT + ".stateChange(e=" + e.toString() + ")");
		if (e.getSource() instanceof JCheckBox) {
			JCheckBox ck = (JCheckBox) e.getSource();
			if (ck.getName().equals(ACT.CK_DISPOSITION.n())) {
				App.preferences.setBoolean(Pref.KEY.MEMO_LAYOUT_DIRECTION, ck.isSelected());
				//setDisposition();
			}
		}
	}

	private Memo getMemo(Long id) {
		return (Memo) EntityUtil.findEntity(mainFrame, Book.TYPE.MEMO, id);
	}

	private void checkTextChanged() {
		if (currentMemoId == null || currentMemoId == -1L) {
			return;
		}
		Memo memo = getMemo(currentMemoId);
		String oldtext = memo.getNotes();
		String nextext = shef.getText();
		shef.setCaretPosition(0);
		if (!oldtext.equals(nextext)) {
			memo.setNotes(shef.getText());
			mainFrame.getBookController().updateEntity(memo);
			mainFrame.setUpdated();
		}
	}
}
