/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.panel.navigation;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconButton;
import storybook.ctrl.Ctrl;
import i18n.I18N;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.hbn.entity.Chapter;
import storybook.tools.swing.SwingUtil;
import storybook.tools.swing.js.JSLabel;
import storybook.tools.swing.js.JSPanelViewsRadioButton;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.SbView.VIEWNAME;
import storybook.ui.panel.AbstractPanel;

/**
 * @author martin
 *
 */
@SuppressWarnings("serial")
public class FindChapterPanel extends AbstractPanel {

	private JComboBox chapterCombo;
	private JSPanelViewsRadioButton viewsRbPanel;

	public FindChapterPanel(MainFrame mainFrame) {
		super(mainFrame);
		initAll();
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		// empty
	}

	@Override
	public void init() {
		// empty
	}

	@Override
	public void initUi() {
		setLayout(new MigLayout(MIG.get(MIG.FILLX, MIG.WRAP + " 2"), "[]10[grow]10[]", "[]10[]10[]"));
		add(new JSLabel(I18N.getColonMsg("chapter")));
		chapterCombo = new JComboBox();
		Chapter chapter = new Chapter();
		EntityUtil.fillCB(mainFrame, chapterCombo, Book.TYPE.CHAPTER, chapter, false, false);
		SwingUtil.setMaxWidth(chapterCombo, 200);
		add(chapterCombo, MIG.get(MIG.GROWX, MIG.SPAN + " 2", MIG.SPLIT + " 3"));
		IconButton btPrev = new IconButton("btPrev", ICONS.K.NAV_PREV, getPreviousAction());
		//btPrev.set20x20();
		add(btPrev);
		IconButton btNext = new IconButton("btNext", ICONS.K.NAV_NEXT, getNextAction());
		//btNext.set20x20();
		add(btNext);
		add(new JSLabel(I18N.getColonMsg("navigation.show.in")), MIG.TOP);
		viewsRbPanel = new JSPanelViewsRadioButton(mainFrame);
		add(viewsRbPanel);
		JButton btFind = new IconButton("btFind", ICONS.K.SEARCH, getFindAction());
		btFind.setText(I18N.getMsg("find"));
		SwingUtil.addEnterAction(btFind, getFindAction());
		add(btFind, MIG.get(MIG.SPAN, MIG.RIGHT));
	}

	private AbstractAction getPreviousAction() {
		return new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				int index = chapterCombo.getSelectedIndex();
				--index;
				if (index == -1) {
					index = 0;
				}
				chapterCombo.setSelectedIndex(index);
				scrollToChapter();
			}
		};
	}

	private AbstractAction getNextAction() {
		return new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				int index = chapterCombo.getSelectedIndex();
				++index;
				if (index == chapterCombo.getItemCount()) {
					index = chapterCombo.getItemCount() - 1;
				}
				chapterCombo.setSelectedIndex(index);
				scrollToChapter();
			}
		};
	}

	private AbstractAction getFindAction() {
		return new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent evt) {
				scrollToChapter();
			}
		};
	}

	private void scrollToChapter() {
		Chapter chapter = (Chapter) chapterCombo.getSelectedItem();
		Ctrl ctrlBook = mainFrame.getBookController();
		if (viewsRbPanel.isChronoSelected()) {
			mainFrame.showView(VIEWNAME.CHRONO);
			ctrlBook.chronoShowEntity(chapter);
		} else if (viewsRbPanel.isBookSelected()) {
			mainFrame.showView(VIEWNAME.BOOK);
			ctrlBook.bookShowEntity(chapter);
		} else if (viewsRbPanel.isManageSelected()) {
			mainFrame.showView(VIEWNAME.MANAGE);
			ctrlBook.manageShowEntity(chapter);
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// empty
	}

}
