/*
 * Copyright (C) 2023 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.review;

import i18n.I18N;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.Toolkit;
import java.io.File;
import java.util.List;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import org.miginfocom.swing.MigLayout;
import resources.icons.ICONS;
import resources.icons.IconUtil;
import storybook.model.EntityUtil;
import storybook.model.book.Book;
import storybook.model.hbn.entity.Endnote;
import storybook.model.hbn.entity.Scene;
import storybook.tools.file.IOUtil;
import storybook.tools.html.CSS;
import storybook.tools.html.Html;
import storybook.tools.net.Net;
import storybook.tools.swing.SwingUtil;
import storybook.tools.swing.js.JSToolBar;
import storybook.ui.MIG;
import storybook.ui.MainFrame;
import storybook.ui.Ui;
import storybook.ui.panel.typist.TypistPanel;

/**
 *
 * @author favdb
 */
public class ReviewPanel extends JPanel implements HyperlinkListener {

	private static final String TT = "ReviewPanel";

	private JEditorPane edComments;
	private Scene scene;
	private List<Endnote> comments;
	private final TypistPanel typist;
	private JButton btExport, btImport;
	private final MainFrame mainFrame;
	private JButton btRemove;

	public ReviewPanel(MainFrame mainFrame, TypistPanel typist) {
		this.mainFrame = mainFrame;
		this.typist = typist;
		this.scene = typist.getScene();
		initialize();
	}

	public void initialize() {
		//LOG.trace(TT + ".initUi()");
		Border bord = BorderFactory.createLineBorder(Color.red);
		setBorder(BorderFactory.createTitledBorder(bord, I18N.getColonMsg("comments")));
		setLayout(new MigLayout(MIG.get(MIG.WRAP + " 2"), "[fill,grow]"));
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		d.width = (d.width / 12) * 9;
		setMaximumSize(d);
		this.add(initToolbar(), MIG.get(MIG.GROWX, MIG.SPAN));
		this.add(initEdComments(), MIG.GROW);
	}

	private JScrollPane initEdComments() {
		edComments = new JEditorPane();
		edComments.setEditable(false);
		edComments.setContentType(Html.TYPE);
		edComments.addHyperlinkListener(this);
		JScrollPane scroller = new JScrollPane(edComments);
		SwingUtil.setMaxPreferredSize(scroller);
		return scroller;
	}

	public JToolBar initToolbar() {
		//LOG.trace(TT + ".initToolbar()");
		JSToolBar tb = new JSToolBar(false);
		tb.setLayout(new MigLayout(MIG.get(MIG.FILLX, MIG.INS0, MIG.HIDEMODE3)));
		tb.setName("CommentsToolbar");
		JButton bten = Ui.initButton("btAdd", "", ICONS.K.CHAR_COMMENT,
				"comment.create",
				evt -> commentAdd());
		bten.setMnemonic('C');
		tb.add(bten);
		/*btRemove = Ui.initButton("btRemove", "", ICONS.K.REMOVE,
				"comments.remove_all",
				evt -> commentsRemove());
		tb.add(btRemove, MIG.LEFT);*/
		tb.add(new JLabel(), MIG.GROWX);
		JPanel p = new JPanel(new MigLayout(MIG.INS0));
		p.setOpaque(false);
		btExport = Ui.initButton("btExport", "", ICONS.K.F_EXPORT,
				"comments.export",
				evt -> commentsExport());
		p.add(btExport);
		btImport = Ui.initButton("btImport", "", ICONS.K.F_IMPORT,
				"comments.import",
				evt -> commentsImport());
		p.add(btImport);
		tb.add(p, MIG.RIGHT);
		return tb;
	}

	private void commentAdd() {
		//LOG.trace(TT + ".commentAdd()");
		Endnote en = Endnote.createEndnote(mainFrame, 1, scene, typist.getHtTexte());
		if (en != null) {
			typist.setModified();
		}
		refresh();
	}

	private void commentEdit(Endnote endnote) {
		//LOG.trace(TT + ".commentEdit(endnote)");
		if (!mainFrame.showEditorAsDialog(endnote)) {
			refresh();
		}
	}

	private void commentsRemove() {
		//LOG.trace(TT + ".commentsRemove()");
		Endnote.removeEndnotes(mainFrame, 1, scene);
		typist.refresh();
	}

	private void commentRemove(Endnote comment) {
		//LOG.trace(TT + ".commentRemove(comment=" + LOG.traceEntity(comment) + ")");
		if (comment != null) {
			String str = Endnote.linkTo("", comment);
			typist.getHtTexte().wysEditorGet().removeTag(str);
			mainFrame.getBookModel().deleteEntity(comment);
			refresh();
		}
	}

	/**
	 * popup menu to select the format to export
	 */
	private void commentsExport() {
		//LOG.trace(TT + ".commentsExport()");
		JPopupMenu menu = new JPopupMenu();
		menu.add(new JLabel(I18N.getMsg("comments.export_to")));
		JMenuItem item;
		item = new JMenuItem("TXT");
		item.addActionListener(e -> ExportReview.exec(mainFrame, this, "txt"));
		menu.add(item);
		item = new JMenuItem("HTML");
		item.addActionListener(e -> ExportReview.exec(mainFrame, this, "html"));
		menu.add(item);
		item = new JMenuItem("XML");
		item.addActionListener(e -> ExportReview.exec(mainFrame, this, "xml"));
		menu.add(item);
		menu.show(btExport, btExport.getX(), btExport.getY());
	}

	/**
	 * select the file to import and select if existing coments	are to be remove
	 */
	private void commentsImport() {
		//LOG.trace(TT + ".commentsImport()");
		typist.saveText();
		//select a file to import (XML only)
		File file = IOUtil.fileSelect(this, "", "xml",
				I18N.getMsg("file.type.xml") + " (*.xml)", "import");
		if (file == null) {
			return;
		}
		// ask if add imported Comments or Replace all
		if (ImportReview.exec(mainFrame, file)) {
			typist.refresh();
			JOptionPane.showMessageDialog(null,
					I18N.getMsg("comments.import.ok", file.getAbsolutePath()),
					I18N.getMsg("import"),
					JOptionPane.INFORMATION_MESSAGE);
		}
	}

	@SuppressWarnings("unchecked")
	public void refresh() {
		//LOG.trace(TT + ".refresh()");
		comments = Endnote.find(mainFrame, 1, scene);
		StringBuilder buf = new StringBuilder(Html.HTML_B);
		buf.append(Html.HEAD_B);
		buf.append(Html.STYLE_B);
		buf.append(CSS.forEditor());
		buf.append(Html.STYLE_E);
		buf.append(Html.HEAD_E);
		buf.append(Html.BODY_B);
		buf.append(Endnote.reviewsToHtml(mainFrame, comments));
		buf.append(Html.BODY_E);
		buf.append(Html.HTML_E);
		edComments.setText(buf.toString());
		edComments.setCaretPosition(0);
	}

	public void save() {
		//LOG.trace(TT + ".save()");
	}

	public void setScene(Scene s) {
		this.scene = s;
		refresh();
	}

	@Override
	public void hyperlinkUpdate(HyperlinkEvent evt) {
		//LOG.trace(TT + ".hyperlinkUpdate(evt=" + evt.toString() + ")");
		String url = evt.getDescription();
		if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
			if (url.toLowerCase().startsWith("http") || url.toLowerCase().startsWith("file")) {
				Net.openUrl(evt);
			} else {
				if (evt.getInputEvent().isControlDown()) {
					Long id = Long.valueOf(url.substring(url.indexOf("_") + 1));
					Endnote endnote = (Endnote) EntityUtil.findEntity(mainFrame, Book.TYPE.ENDNOTE, id);
					JPopupMenu menu = new JPopupMenu();
					JMenuItem item;
					item = new JMenuItem(I18N.getMsg("edit"));
					item.setIcon(IconUtil.getIconSmall(ICONS.K.EDIT));
					item.addActionListener(e -> commentEdit(endnote));
					menu.add(item);
					if (!comments.isEmpty()) {
						item = new JMenuItem(I18N.getMsg("comment.remove"));
						item.setIcon(IconUtil.getIconSmall(ICONS.K.REMOVE));
						item.addActionListener(e -> commentRemove(endnote));
						menu.add(item);
						item = new JMenuItem(I18N.getMsg("comments.remove_scene"));
						item.setIcon(IconUtil.getIconSmall(ICONS.K.REMOVE));
						item.addActionListener(e -> commentsRemove());
						menu.add(item);
					}
					Point p = MouseInfo.getPointerInfo().getLocation();
					SwingUtilities.convertPointFromScreen(p, edComments);
					menu.show(edComments, p.x, p.y);
				}
			}
		}
	}

}
